/*
 * Glacier - another Gnome email client.
 * Copyright (C) 1999 Lubomir Gelo
 *
 * Glacier uses code from other GPL-ed projects. See THANKS for 
 * credits and copyright info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef __MAILBOX_H__
#define __MAILBOX_H__

typedef enum {

    MAILBOX_MBOX,
    MAILBOX_FOLDER,
    MAILBOX_LOCAL,
    MAILBOX_INBOX,
    MAILBOX_OUTBOX,
    MAILBOX_TRASH,
    MAILBOX_UNKNOWN

} MailboxType;

typedef struct _Mailbox		Mailbox;
typedef struct _MailboxNode 	MailboxNode;

typedef struct _Message		Message;
typedef struct _Envelope	Envelope;
typedef struct _Header		Header;
typedef struct _Body		Body;
typedef struct _Content		Content;
typedef struct _Parameter	Parameter;
typedef struct _Address		Address;

struct _Address {

    char *full;		
    char *person;
    char *mailbox;
    gint group;
};

struct _Mailbox {

    MailboxType 	type;
    gchar		*name;
    gchar		*path;

    GList		*messages;

    FILE 		*fp;
    int 		locked;
    gboolean 		readonly;
    int 		size;

    time_t		mtime;

    int			total_messages;
    int			new_messages;
    
    MailboxNode		*mbnode;
    
};

struct _MailboxNode {

    Mailbox 		*mailbox;
    gint 		expanded;

    GList		*children;
    MailboxNode 	*parent;
    GtkCTreeNode	*ctree_node;
};

struct _Message {

    Mailbox 		*mailbox;		/* the mailbox this message belongs to */
    Envelope 		*envelope;
    Header		*header;
    Body   		*content;		/* contains body parts of message */
};

struct _Envelope {

    GList		*return_path;
    GList		*from;
    GList		*to;
    GList		*cc;
    GList		*bcc;
    GList		*sender;
    GList		*reply_to;
    GList		*mail_followup_to;

    gchar 		*subject;
    gchar 		*real_subj;		/* offset of the real subject */
    gchar 		*message_id;
    gchar		*supersedes;

    GList 		*references;		/* message references (in reverse order) */
    GList 		*userhdrs;		/* user defined headers */
};

struct _Parameter {

    char 		*attribute;
    char 		*value;
};

/* Information that helps in determing the Content-* of an attachment */
struct _Content {

    long 		hibin;              	/* 8-bit characters */
    long 		lobin;              	/* unprintable 7-bit chars (eg., control chars) */
    long 		ascii;              	/* number of ascii chars */
    long 		linemax;        	/* length of the longest line in the file */
    unsigned int 	space : 1;  		/* whitespace at the end of lines? */
    unsigned int 	binary : 1; 		/* long lines, or CR not in CRLF pair */
    unsigned int 	from : 1;   		/* has a line beginning with "From "? */
    unsigned int 	dot : 1;    		/* has a line consisting of a single dot? */
};

struct _Body {

    char 		*xtype;	 		/* content-type if x-unknown */
    char 		*subtype;               /* content-type subtype */
    GList 		*parameter;       	/* parameters of the content-type */
    char 		*description;           /* content-description */

    long		hdr_offset;
    long 		offset;                 /* offset where the actual data begins */
    long 		length;                 /* length (in bytes) of attachment */
    Content 		*content;           	/* structure used to store detailed info about
						 * the content of the attachment.  this is used
						 * to determine what content-transfer-encoding
						 * is required when sending mail.
						 */
    gchar		*filename;
    gchar		*d_filename;
    gchar		*form_name;

    Body		*next;
    GList 		*parts;       		/* parts of a multipart or message/rfc822 */
    time_t 		stamp;			/* time stamp of last encoding update. */
  
    unsigned int 	type : 3;        	/* content-type primary type */
    unsigned int 	encoding : 3;    	/* content-transfer-encoding */
    unsigned int 	disposition : 2; 	/* content-disposition */
    unsigned int 	use_disp : 1;    	/* Content-Disposition field printed? */

    unsigned int 	tagged : 1;
    unsigned int 	deleted : 1;		/* attachment marked for deletion */
};

struct _Header {

    unsigned int 	mime : 1;    		/* has a Mime-Version header? */
    unsigned int 	flagged : 1; 		/* marked important? */
    unsigned int 	tagged : 1;
    unsigned int 	deleted : 1;
    unsigned int 	changed : 1;
    unsigned int 	attach_del : 1; 	/* has an attachment marked for deletion */
    unsigned int 	old : 1;
    unsigned int 	read : 1;
    unsigned int 	expired : 1; 		/* already expired? */
    unsigned int 	superseded : 1; 	/* got superseded? */
    unsigned int 	replied : 1;

    /* timezone of the sender of this message */
    unsigned int 	zhours : 5;
    unsigned int 	zminutes : 6;
    unsigned int 	zoccident : 1;

    time_t 		date_sent;     		/* time when the message was sent (UTC) */
    time_t 		received;      		/* time when the message was placed in the mailbox */
    long 		offset;        		/* where in the stream does this message begin? */
    int 		lines;			/* how many lines in the body of this message? */
    int 		index;			/* the absolute (unsorted) message number */
    int 		msgno;			/* number displayed to the user */
    int 		virtual;		/* virtual message number */
};

typedef struct {
    FILE *fpin;
    FILE *fpout;
    gchar *prefix;
    int flags;
} State;


#endif /* __MAILBOX_H__ */