/*
 * Glacier - another Gnome email client.
 * Copyright (C) 1999 Lubomir Gelo
 *
 * Glacier uses code from other GPL-ed projects. See THANKS for 
 * credits and copyright info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "includes.h"

#define GLACIER_APPBAR_KEY  "glacier_appbar"

#ifdef DEBUG
gint debuglevel	= 100;
#endif

/* GLOBALS */
GlacierApp	glacier_app;

static guint pbar_timeout;

/* internal functions */
static void     init_application(void);
static gint 	progress_timeout (gpointer data);
void 		dialog_not_implemented(void);
static void 	dialog_welcome(void);

/* menu handlers */
static void     on_menu_about(GtkWidget *widget, void *data);
static void     on_menu_quit(GtkWidget *widget, void *data);
static void     on_menu_not_implemented (GtkWidget *widget, void *data);
static void 	on_menu_preferences (GtkWidget *widget, void *data);

static void 	on_toolbar_prev_message(GtkWidget * widget, gpointer data);
static void 	on_toolbar_next_message(GtkWidget * widget, gpointer data);

/* temp */
GlacierMsgview *glacier_create_message_preview (GtkWidget *paned, int position);
GtkWidget *glacier_create_message_list(GtkWidget *paned, int position);

void gtk_widget_get_usize (GtkWidget *widget, gint *width, gint *height);


/* menu structures */
static GnomeUIInfo help_menu [] = {

    GNOMEUIINFO_ITEM_STOCK (N_("About Glacier"), NULL, on_menu_about, GNOME_STOCK_MENU_ABOUT), GNOMEUIINFO_END
};

static GnomeUIInfo mailbox_menu[] = {

    {
	GNOME_APP_UI_ITEM, N_("_Add"), N_("_Add new mailbox"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    {
	GNOME_APP_UI_ITEM, N_("_Edit"), N_("_Edit mailbox properties"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    {
	GNOME_APP_UI_ITEM, N_("_Delete"), N_("_Delete mailbox"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    GNOMEUIINFO_END
};

static GnomeUIInfo message_menu[] = {

    {
	GNOME_APP_UI_ITEM, N_("_New"), N_("_Create new message"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    {
	GNOME_APP_UI_ITEM, N_("_Reply"), N_("_Send reply to message sender"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    {
	GNOME_APP_UI_ITEM, N_("_Reply to all"), N_("_Send reply to all recipients"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    {
	GNOME_APP_UI_ITEM, N_("_Forward"), N_("_Forward message"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    GNOMEUIINFO_SEPARATOR,

    {
	GNOME_APP_UI_ITEM, N_("_Delete"), N_("_Delete message"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
	GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },


    GNOMEUIINFO_END
};


static GnomeUIInfo settings_menu[] = {

    {
	GNOME_APP_UI_ITEM, N_ ("_Preferences"), N_("Configure the application"),
	on_menu_preferences, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
        GNOME_STOCK_MENU_MAIL_RCV, 0, GDK_CONTROL_MASK, NULL
    },

    GNOMEUIINFO_END

};

static GnomeUIInfo file_menu[] = {

    {
	GNOME_APP_UI_ITEM, N_ ("_Check Mail"), N_("Scan mailboxes for new mail"),
	on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
        GNOME_STOCK_MENU_MAIL_RCV, 'M', GDK_CONTROL_MASK, NULL
    },

    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_EXIT_ITEM(on_menu_quit, NULL),
    GNOMEUIINFO_END
};


static GnomeUIInfo main_menu [] = {

    GNOMEUIINFO_MENU_FILE_TREE(file_menu),
    GNOMEUIINFO_SUBTREE(N_("Message"), &message_menu),
    GNOMEUIINFO_SUBTREE(N_("Mailbox"), &mailbox_menu),
    GNOMEUIINFO_SUBTREE(N_("Settings"), &settings_menu),
    GNOMEUIINFO_SUBTREE (N_("Help"), &help_menu),
    GNOMEUIINFO_END
};



/* The toolbar */
static GnomeUIInfo toolbar_info[] = {

    { GNOME_APP_UI_ITEM, "Check", "Check Email",on_menu_not_implemented , NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_RCV, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Delete", "Delete Message", on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TRASH, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Compose", "Compose Message", on_menu_not_implemented, "new", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_NEW, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Reply", "Reply", on_menu_not_implemented, "reply", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_RPL, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Reply to all", "Reply to all", on_menu_not_implemented, "replytoall", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_RPL, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Forward", "Forward", on_menu_not_implemented, "forward", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_MAIL_FWD, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Prev Msg", "Prev Msg", on_toolbar_prev_message, "forward", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_BACK, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Next Msg", "Next Msg", on_toolbar_next_message, "forward", NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_FORWARD, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Print", "Print", on_menu_not_implemented, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_PRINT, 0, 0, NULL },
    { GNOME_APP_UI_ITEM, "Quit", "Quit", on_menu_quit, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_QUIT, 0, 0, NULL },
    GNOMEUIINFO_END
};



GlacierMsgview *glacier_create_message_preview (GtkWidget *paned, int position){

    GtkWidget 		*scrolled, *frame;
    GlacierMsgview 	*view;
    GtkAdjustment 	*vadj, *hadj;

    scrolled = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
    
    view = msgview_new ();
    gtk_widget_set_usize(GTK_WIDGET(view), glacier_app.msgview_width, glacier_app.msgview_height);	

    gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET(view));
    gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET(frame));

    gtk_container_set_border_width (GTK_CONTAINER (scrolled), 5);
      
    vadj = gtk_layout_get_vadjustment(GTK_LAYOUT(view));
    hadj = gtk_layout_get_hadjustment(GTK_LAYOUT(view));
//    vadj->step_increment = 10;
//    hadj->step_increment = 10;

    gtk_scrolled_window_set_vadjustment( GTK_SCROLLED_WINDOW(scrolled), vadj);
    gtk_scrolled_window_set_hadjustment( GTK_SCROLLED_WINDOW(scrolled), hadj);

    if (position == 1)
	gtk_paned_add1 (GTK_PANED(paned), scrolled);
    else
	gtk_paned_add2 	(GTK_PANED(paned), scrolled);

    gtk_widget_show(GTK_WIDGET(view));
    gtk_widget_show(frame);
    gtk_widget_show(scrolled);

    return view;

/*
    GdkFont 	*fixed_font;
    view = gnome_less_new ();
    fixed_font = gdk_font_load ("-adobe-helvetica-medium-r-normal-*-*-180-*-*-p-*-iso8859-1");

    gtk_container_set_border_width (GTK_CONTAINER (view), 5);
    gtk_widget_set_usize(view, glacier_app.msgview_width, glacier_app.msgview_height);	

    if (position == 1)
	gtk_paned_add1 (GTK_PANED(paned), view);
    else
	gtk_paned_add2 	(GTK_PANED(paned), view);

    gtk_widget_show (view);

    return view;
*/
}

/* everything starts here */
int main (int argc, char *argv[]) {
	
    gnome_init (PACKAGE, VERSION, argc, argv);

    memset(&glacier_app, 0, sizeof(GlacierApp));

    /* Read settings. */
    prefs_load ();

    /* load icons */
    icons_init();

    /* global init */	
    init_application ();

    if (prefs_check_config_version() == GLACIER_NOT_CONFIGURED)
	dialog_welcome();
	
    gtk_main ();
	
    return 0;
}

static void init_application(void){

    GtkWidget *app;

    /* create app window */
    app = gnome_app_new ("glacier", "Glacier");
    glacier_app.app = GTK_WINDOW(app);

    gtk_signal_connect (GTK_OBJECT (app), "delete_event", GTK_SIGNAL_FUNC (on_menu_quit),  NULL);

    gtk_window_set_policy (GTK_WINDOW (glacier_app.app), TRUE, TRUE, TRUE);
    gtk_widget_set_usize(GTK_WIDGET(app), glacier_app.app_width, glacier_app.app_height);	

    /* menu & toolbar */
    gnome_app_create_menus (GNOME_APP (app), main_menu);
    gnome_app_create_toolbar (GNOME_APP (app), toolbar_info); 

    /* statusbar */
    glacier_app.status_bar = GNOME_APPBAR(gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_USER));
    gnome_app_set_statusbar (GNOME_APP (app), GTK_WIDGET (glacier_app.status_bar));
    gtk_object_set_data (GTK_OBJECT (app), GLACIER_APPBAR_KEY, glacier_app.status_bar);


    switch (glacier_app.layout){

	case APP_LAYOUT_U:

	    glacier_app.vpaned = gtk_vpaned_new();
	    gnome_app_set_contents(GNOME_APP (app), glacier_app.vpaned);

	    glacier_app.hpaned = gtk_hpaned_new();
	    gtk_paned_add1(GTK_PANED(glacier_app.vpaned), glacier_app.hpaned);
	    
	    glacier_app.mailbox_tree = glacier_mbtree_new (glacier_app.hpaned, 1);
	    glacier_app.message_list = glacier_create_message_list(glacier_app.hpaned, 2);
	    glacier_app.message_preview =  glacier_create_message_preview (glacier_app.vpaned, 2);
	    break;


	case APP_LAYOUT_C:
	    glacier_app.hpaned = gtk_hpaned_new();
	    gnome_app_set_contents(GNOME_APP (app), glacier_app.hpaned);
	    glacier_app.vpaned = gtk_vpaned_new();

	    glacier_app.mailbox_tree = glacier_mbtree_new (glacier_app.hpaned, 1);
	    glacier_app.message_list = glacier_create_message_list(glacier_app.vpaned, 1);
	    glacier_app.message_preview =  glacier_create_message_preview (glacier_app.vpaned, 2);

	    gtk_paned_add2(GTK_PANED(glacier_app.hpaned), glacier_app.vpaned);
	    break;

	case APP_LAYOUT_I:
	default:
	    glacier_app.vpaned = NULL;
	    glacier_app.message_preview=NULL;
	    glacier_app.hpaned = gtk_hpaned_new();
	    gnome_app_set_contents(GNOME_APP (app), glacier_app.hpaned);

	    glacier_app.mailbox_tree = glacier_mbtree_new (glacier_app.hpaned, 1);
	    glacier_app.message_list = glacier_create_message_list(glacier_app.hpaned, 2);
	    break;

    }

    if (glacier_app.hpaned){
	gtk_paned_set_handle_size (GTK_PANED(glacier_app.hpaned), 10);
	gtk_paned_set_gutter_size (GTK_PANED(glacier_app.hpaned), 5);
	gtk_widget_show (glacier_app.hpaned);
    }    

    if (glacier_app.vpaned){
	gtk_paned_set_handle_size (GTK_PANED(glacier_app.vpaned), 10);
	gtk_paned_set_gutter_size (GTK_PANED(glacier_app.vpaned), 5);
	gtk_widget_show (glacier_app.vpaned);
    }    

    gtk_widget_show (app);
    glacier_mbtree_load_mailboxes();

}

void set_cursor (gint type){

//    GdkCursor *cursor;
    GtkProgress *pbar;

    pbar = gnome_appbar_get_progress(glacier_app.status_bar);
      
    if (type == -1) {
	gtk_widget_set_sensitive (GTK_WIDGET (pbar), FALSE);
        gtk_progress_set_activity_mode (GTK_PROGRESS (pbar), FALSE);
        gtk_timeout_remove (pbar_timeout);
        gtk_progress_set_value (GTK_PROGRESS (pbar), 0.0);
//	gdk_window_set_cursor (GTK_WIDGET(glacier_app.app)->window, NULL);
    } else {
        gtk_widget_set_sensitive (GTK_WIDGET (pbar), TRUE);
        gtk_progress_set_activity_mode (GTK_PROGRESS (pbar), TRUE);
        pbar_timeout = gtk_timeout_add (50, progress_timeout, pbar);
//	cursor = gdk_cursor_new (type);
//	gdk_window_set_cursor (GTK_WIDGET(glacier_app.appb)->window, cursor);
//	gdk_cursor_destroy (cursor);
    }
}

static gint progress_timeout (gpointer data){

    GtkProgress *pbar;
    GtkAdjustment *adj;
    gfloat new_val;

    pbar = GTK_PROGRESS(data);
    adj = pbar->adjustment;

    new_val = adj->value + 1;

    if (new_val > adj->upper)
	new_val = adj->lower;

    gtk_progress_set_value (pbar, new_val);

    return TRUE;
}


void dialog_not_implemented (void) {

    GtkWidget *dialog;

    dialog = gnome_message_box_new("Sorry not implemented", GNOME_MESSAGE_BOX_ERROR, "OK", 0);
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
    gtk_widget_show(dialog);
}

/* main menu callbacks */
static void on_menu_quit(GtkWidget *widget, void *data){

    glacier_mbtree_close_mailboxes(glacier_app.mailbox_list);

    if (glacier_app.current_mailbox){
	mbox_close_mailbox(glacier_app.current_mailbox);
	glacier_app.current_mailbox = NULL;
    }

    gtk_widget_get_usize(GTK_WIDGET(glacier_app.app), &glacier_app.app_width, &glacier_app.app_height);
    gtk_widget_get_usize(GTK_WIDGET(glacier_app.message_list), &glacier_app.msglist_width, &glacier_app.msglist_height);
    gtk_widget_get_usize(GTK_WIDGET(glacier_app.mailbox_tree), &glacier_app.mbtree_width, &glacier_app.mbtree_height);
    gtk_widget_get_usize(GTK_WIDGET(glacier_app.message_preview), &glacier_app.msgview_width, &glacier_app.msgview_height);

    glacier_app.mbtree_width +=14;
    glacier_app.msglist_height += 10;
    glacier_app.msgview_height -= 10;

    prefs_save();



    gtk_main_quit ();
}

static void on_menu_about (GtkWidget *widget, void *data){

    GtkWidget *about;
    const gchar *auth[] = {"Lubomir Gelo (slon@slon.sk)", NULL};
	
    about = gnome_about_new ( _("Glacier"), VERSION, _("Copyright (C) 1999 Lubomir Gelo"),
			      auth, _("Another Gnome email client.\nHomepage: http://glacier.linuxbox.com"), NULL);

    gtk_widget_show (about);
}

static void on_menu_not_implemented (GtkWidget *widget, void *data){

    dialog_not_implemented();
}

static void on_menu_preferences (GtkWidget *widget, void *data){

    prefs_dialog_open();
}

static void on_toolbar_next_message(GtkWidget * widget, gpointer data){

    g_return_if_fail (widget != NULL);

    if (!glacier_app.current_mailbox)
	return;

    msglist_select_next(GTK_CLIST(glacier_app.message_list));
}


static void on_toolbar_prev_message(GtkWidget * widget, gpointer data){

    g_return_if_fail (widget != NULL);

    if (!glacier_app.current_mailbox)
	return;

    msglist_select_prev(GTK_CLIST(glacier_app.message_list));
}




void gtk_widget_get_usize (GtkWidget *widget, gint *width, gint *height){

    g_return_if_fail (widget != NULL);
    g_return_if_fail (GTK_IS_WIDGET (widget));

    *width = (widget->allocation).width;
    *height = (widget->allocation).height;
}

static void dialog_welcome (void) {

    GtkWidget *dialog;

    dialog = gnome_message_box_new("Welcome to Glacier.\nIt seems this is the first GLACIER run.\nPlease do not forget to edit your preferences. ", GNOME_MESSAGE_BOX_INFO, "OK", 0);
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
    gtk_widget_show(dialog);
}

/*
void msgview_save_attachment_dialog(void){

    GtkWidget *dialog, *vbox, *pixmap, *label_type, *label_size;

    dialog = gnome_dialog_new("Save Attachment", GNOME_STOCK_BUTTON_CANCEL, GNOME_STOCK_BUTTON_OK, NULL);    
    gtk_widget_set_usize (dialog, 300, 300);
    gtk_object_set_data(GTK_OBJECT(dialog), "dialog", (gpointer)&dialog);
    gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, FALSE);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_object_set_data(GTK_OBJECT(dialog), "vbox", vbox);
    gtk_widget_show(vbox);
    gtk_container_add(GTK_CONTAINER(dialog), vbox);

//    pixmap = create_pixmap(dalog, "src/attachment.xpm");
    if (pixmap == NULL)
	g_error("Couldn't create pixmap");

//    gtk_object_set_data(GTK_OBJECT(dialog), "pixmap", pixmap);
//    gtk_widget_show(pixmap);
//    gtk_box_pack_start (GTK_BOX(vbox), pixmap, TRUE, TRUE, 0);

    label_type = gtk_label_new ("application/gnome");
    gtk_object_set_data (GTK_OBJECT (dialog), "label_type", label_type);
    gtk_widget_show (label_type);
    gtk_box_pack_start (GTK_BOX (vbox), label_type, TRUE, TRUE, 0);

    label_size = gtk_label_new ("12.3 kB");
    gtk_object_set_data (GTK_OBJECT (dialog), "label_size", label_size);
    gtk_widget_show (label_size);
    gtk_box_pack_start (GTK_BOX (vbox), label_size, TRUE, TRUE, 0);

    gtk_widget_show(dialog);
    gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
    gnome_dialog_run(GNOME_DIALOG(dialog));
    gtk_widget_destroy(dialog);

}
*/