/*
 * Glacier - another Gnome email client.
 * Copyright (C) 1999 Lubomir Gelo
 *
 * Glacier uses code from other GPL-ed projects. See THANKS for 
 * credits and copyright info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "includes.h"

struct glacier_file_info {

    gchar path[PATH_MAX];
    gint  isDir;		

};

enum  {
    SELECT_MAILBOX,
    LAST_SIGNAL
};

/* object arguments */
enum {
    ARG_0,
    ARG_SHOW_CONTENT_INFO
};

static gint glacier_mbtree_signals[LAST_SIGNAL] = {0};
static GlacierMbTreeClass *parent_class = NULL;


static guint glacier_mbtree_get_type (void);
static void glacier_mbtree_destroy (GtkObject * obj);
static void glacier_mbtree_class_init (GlacierMbTreeClass *obj_class);
static void glacier_mbtree_set_arg (GtkObject * object, GtkArg * arg, guint arg_id);
static void glacier_mbtree_get_arg (GtkObject * object, GtkArg * arg, guint arg_id);
static void glacier_mbtree_init(GlacierMbTree * tree);
static void glacier_mbtree_redraw (GlacierMbTree *tree);
static void glacier_mbtree_select (GtkCTree * ctree, GtkCTreeNode *row, gint column);

static gint glacier_mbtree_on_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data);

static GtkWidget *glacier_mbtree_create_context_menu (GtkCTree * ctree, MailboxNode *mbnode);

static void glacier_mbtree_on_menu_open (GtkWidget * widget, MailboxNode *mbnode);
static void glacier_mbtree_on_menu_delete (GtkWidget * widget, MailboxNode *mbnode);
static void glacier_mbtree_on_menu_open (GtkWidget * widget, MailboxNode *mbnode);
static void glacier_mbtree_on_menu_add (GtkWidget * widget, MailboxNode *mbnode);
static void glacier_mbtree_on_menu_edit (GtkWidget * widget, MailboxNode *mbnode);

static void glacier_mbtree_expand (GtkCTree *, GtkCTreeNode *, gpointer);
static void glacier_mbtree_collapse (GtkCTree *, GtkCTreeNode *, gpointer);

static gboolean mailbox_is_special(gchar *path);

static void glacier_dialog_mailbox_add_mailbox (MailboxNode *mbnode);
static void glacier_dialog_mailbox_conf_delete (MailboxNode *mbnode);

static void glacier_mailbox_create(MailboxNode *parent_mbnode, gchar *name, gint folder);
static void glacier_mailbox_delete(MailboxNode *mbnode);

static void glacier_mbtree_read_dir(gchar *path, MailboxNode *parent);

static Mailbox *glacier_mailbox_new (MailboxType type);
static void glacier_mailbox_free (Mailbox *mailbox);
static void glacier_mailbox_node_free (MailboxNode *mbnode);
static MailboxNode *glacier_mailbox_node_new (Mailbox * mailbox);
static GList *glacier_mailbox_find_in_list(MailboxNode *mbnode);

static gint glacier_compare_mailboxes(gconstpointer a, gconstpointer b);
static void glacier_mbtree_scan_mailboxes(GList *list);

static guint glacier_mbtree_get_type (void){

    static guint mbtree_type = 0;

    if (!mbtree_type) {

	GtkTypeInfo mbtree_info = { "GlacierMbTree",
				    sizeof (GlacierMbTree),
				    sizeof (GlacierMbTreeClass),
				    (GtkClassInitFunc) glacier_mbtree_class_init,
				    (GtkObjectInitFunc) glacier_mbtree_init,
				    (GtkArgSetFunc) NULL,
				    (GtkArgGetFunc) NULL,
                                  };

	mbtree_type = gtk_type_unique (gtk_ctree_get_type (), &mbtree_info);
    }

    return mbtree_type;
}

GlacierMbTree *glacier_mbtree_new(GtkWidget *paned, int position){

    GlacierMbTree *tree;
    GtkWidget *scrolled;

    scrolled = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);    
    gtk_widget_set_usize(GTK_WIDGET(scrolled), glacier_app.mbtree_width, glacier_app.mbtree_height);	

    tree = gtk_type_new (glacier_mbtree_get_type ());

    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled), GTK_WIDGET(tree));
    gtk_container_set_border_width (GTK_CONTAINER (scrolled), 5);
    
    gtk_clist_set_shadow_type(GTK_CLIST(tree), GTK_SHADOW_NONE);
    gtk_ctree_set_line_style (GTK_CTREE(tree), GTK_CTREE_LINES_DOTTED);
    
    if (position == 1)
	gtk_paned_add1 (GTK_PANED(paned), scrolled);
    else
	gtk_paned_add2 	(GTK_PANED(paned), scrolled);

    gtk_widget_show (GTK_WIDGET(tree));
    gtk_widget_show (scrolled);

    return tree;
}

static void glacier_mbtree_destroy (GtkObject * obj){

    GlacierMbTree *tree;
    tree = GLACIER_MBTREE (obj);

    if (GTK_OBJECT_CLASS (parent_class)->destroy)
	(*GTK_OBJECT_CLASS (parent_class)->destroy) (GTK_OBJECT (tree));

}


static void glacier_mbtree_class_init (GlacierMbTreeClass *obj_class){

    GtkObjectClass *object_class;
    GtkCTreeClass *tree_class;

    object_class = (GtkObjectClass *) obj_class;
    tree_class = GTK_CTREE_CLASS (obj_class);

    glacier_mbtree_signals[SELECT_MAILBOX] =  
				gtk_signal_new ("select_mailbox",
						GTK_RUN_LAST,
						object_class->type,
						GTK_SIGNAL_OFFSET (GlacierMbTreeClass, glacier_mbtree_select),
						gtk_marshal_NONE__POINTER_POINTER_POINTER,
						GTK_TYPE_NONE, 3, GTK_TYPE_POINTER,
						GTK_TYPE_POINTER, GTK_TYPE_GDK_EVENT);

    gtk_object_class_add_signals (object_class, glacier_mbtree_signals, LAST_SIGNAL);

    object_class->destroy	= glacier_mbtree_destroy;
    parent_class 		= gtk_type_class (gtk_ctree_get_type ());
    object_class->set_arg 	= glacier_mbtree_set_arg;
    object_class->get_arg 	= glacier_mbtree_get_arg;

    gtk_object_add_arg_type ("GlacierMbTree::show_content_info", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_SHOW_CONTENT_INFO);

    obj_class->glacier_mbtree_select = NULL;
}


static void glacier_mbtree_set_arg (GtkObject * object, GtkArg * arg, guint arg_id){

}

static void glacier_mbtree_get_arg (GtkObject * object, GtkArg * arg, guint arg_id){

}


static void glacier_mbtree_init(GlacierMbTree * tree){

    char *titles[3] =  {"mailbox", "unread", "total"};

    gtk_widget_push_visual (gdk_imlib_get_visual ());
    gtk_widget_push_colormap (gdk_imlib_get_colormap ());
    gtk_ctree_construct (GTK_CTREE (tree), 1, 0, titles);

    gtk_clist_column_titles_hide (GTK_CLIST (tree));

    gtk_widget_pop_colormap ();
    gtk_widget_pop_visual ();

    gtk_signal_connect (GTK_OBJECT (tree), "tree_expand", GTK_SIGNAL_FUNC (glacier_mbtree_expand), NULL);
    gtk_signal_connect (GTK_OBJECT (tree), "tree_collapse", GTK_SIGNAL_FUNC (glacier_mbtree_collapse), NULL);

    gtk_ctree_set_show_stub (GTK_CTREE (tree), FALSE);
    gtk_ctree_set_line_style (GTK_CTREE (tree), GTK_CTREE_LINES_DOTTED);
    gtk_ctree_set_expander_style (GTK_CTREE (tree), GTK_CTREE_EXPANDER_CIRCULAR);
    gtk_clist_set_row_height (GTK_CLIST (tree), 16);
    gtk_clist_set_column_width (GTK_CLIST (tree), 0, 80);

    gtk_signal_connect (GTK_OBJECT (tree), "tree_select_row", GTK_SIGNAL_FUNC (glacier_mbtree_select), (gpointer) NULL);
    gtk_signal_connect (GTK_OBJECT (tree), "button_press_event", (GtkSignalFunc) glacier_mbtree_on_button_pressed, (gpointer) NULL);

}

static void glacier_mbtree_load_ctree(GlacierMbTree *tree, GList *list){

    MailboxNode *mbnode;
    Mailbox	*mailbox;
    GList	*mblist;
    GtkCTree	*ctree;
    GtkCTreeNode *parent = NULL;
    GtkCTreeNode *sibling = NULL;
    gboolean	is_leaf;
    gchar	*columns[3];
    GdkPixmap    *pixmap_closed, *mask_closed, *pixmap_opened, *mask_opened;
    GtkStyle 	*style = NULL;


    mblist = g_list_first(list);
    ctree = GTK_CTREE (tree);
    
    mbnode = (MailboxNode *)((MailboxNode *)(mblist->data))->parent;

    if (mbnode)
	parent = mbnode->ctree_node;

    while (mblist){
        mbnode = (MailboxNode *)(mblist->data);
	mailbox = (Mailbox*)(mbnode->mailbox);
	    
	columns[0] = mailbox->name;
	columns[1] = NULL;
	columns[2] = NULL;

	is_leaf = mbnode->children == NULL ? TRUE : FALSE;

        if (mbnode->mailbox->new_messages){

	    GdkFont *font;

	    style = gtk_style_copy (gtk_widget_get_style (GTK_WIDGET (ctree)));
            gdk_font_unref(style->font);
	    font = gdk_font_load ("-adobe-helvetica-bold-r-*-*-*-120-*-*-*-*-iso8859-1");
	    style->font = font;
            gdk_font_ref(style->font);

	}

	switch (mbnode->mailbox->type){
	    
	    case MAILBOX_MBOX:

		if (mbnode->mailbox->new_messages){
		    pixmap_closed = icon_get_pixmap (ICON_TRAY_FULL);
		    mask_closed = icon_get_bitmap (ICON_TRAY_FULL);
		} else {
		    pixmap_closed = icon_get_pixmap (ICON_TRAY_EMPTY);
		    mask_closed = icon_get_bitmap (ICON_TRAY_EMPTY);
		}
		pixmap_opened = NULL;
		mask_opened =  NULL;
		break;


	    case MAILBOX_INBOX:

		pixmap_closed = icon_get_pixmap (ICON_INBOX);
		mask_closed = icon_get_bitmap (ICON_INBOX);
		pixmap_opened = NULL;
		mask_opened =  NULL;
		break;


	    case MAILBOX_OUTBOX:

		pixmap_closed = icon_get_pixmap (ICON_OUTBOX);
		mask_closed = icon_get_bitmap (ICON_OUTBOX);
		pixmap_opened = NULL;
		mask_opened =  NULL;
		break;


	    case MAILBOX_TRASH:

		pixmap_closed = icon_get_pixmap (ICON_TRASH);
		mask_closed = icon_get_bitmap (ICON_TRASH);
		pixmap_opened = NULL;
		mask_opened =  NULL;
		break;


	    case MAILBOX_FOLDER:

		pixmap_closed = icon_get_pixmap (ICON_DIR_CLOSED);
		mask_closed = icon_get_bitmap (ICON_DIR_CLOSED);
		pixmap_opened = icon_get_pixmap (ICON_DIR_OPEN);
		mask_opened = icon_get_bitmap (ICON_DIR_OPEN);
		break;

	    default:
		pixmap_closed = NULL;
		mask_closed =  NULL;
		pixmap_opened = NULL;
		mask_opened =  NULL;
		break;

	}

	sibling = mbnode->ctree_node = gtk_ctree_insert_node(	ctree, parent, sibling, columns, 5, 
								pixmap_closed, mask_closed, pixmap_opened, mask_opened,
								is_leaf, mbnode->expanded);

        gtk_ctree_node_set_row_data (ctree, sibling, mbnode);		

        if (mbnode->mailbox->new_messages)
	    gtk_ctree_node_set_row_style (ctree, sibling, style);
								
	if (mbnode->children)
	    glacier_mbtree_load_ctree(tree, mbnode->children);

	mblist = g_list_next(mblist);
    }
}


static void glacier_mbtree_scan_mailboxes(GList *list){

    MailboxNode *mbnode, *parent;
    GList	*mblist;

    mblist = g_list_first(list);

    while (mblist){

        mbnode = (MailboxNode *)(mblist->data);

	if (mbnode->mailbox->type != MAILBOX_FOLDER){

	    mbox_quick_scan(mbnode->mailbox);
	    dprint(99, (stderr, "Scanning %s:  %d total, %d new messags\n", mbnode->mailbox->path, mbnode->mailbox->total_messages, mbnode->mailbox->new_messages));
	    
	    if (mbnode->mailbox->new_messages){

	        parent = (MailboxNode *)((MailboxNode *)(mblist->data))->parent; 	    
	    
		while (parent){
	    
		    if (parent->parent != NULL)
			((Mailbox *)(parent->mailbox))->new_messages = 1;
		    else
			break;
		    
		    parent = parent->parent;
		}		    
		
	    }
	}	    

	if (mbnode->children)
	    glacier_mbtree_scan_mailboxes(mbnode->children);

	mblist = g_list_next(mblist);
    }
}

static void glacier_mbtree_redraw (GlacierMbTree *tree){

    GtkCTree *ctree;

    if (!GLACIER_IS_MBTREE (tree))
	return;

    ctree = GTK_CTREE (tree);
    gtk_clist_freeze (GTK_CLIST (ctree));
    gtk_clist_clear (GTK_CLIST (ctree));

    if (glacier_app.mailbox_list){
         glacier_mbtree_scan_mailboxes(glacier_app.mailbox_list);
	glacier_mbtree_load_ctree(tree, glacier_app.mailbox_list);
    } else {
	fprintf(stderr, "Mailbox list is NULL\n");
    }

    gtk_clist_thaw (GTK_CLIST (ctree));
}

static void glacier_mbtree_select (GtkCTree * ctree, GtkCTreeNode *row, gint column){

    GlacierMbTree *tree;
    GdkEventButton *bevent = (GdkEventButton *) gtk_get_current_event ();
    MailboxNode *mbnode;
    Mailbox *mailbox;

    tree = GLACIER_MBTREE (ctree);
    mbnode = gtk_ctree_node_get_row_data (ctree, row);
    
    if (mbnode) {
	mailbox = mbnode->mailbox;
    	
        if (bevent && bevent->button == 1) {
	    if (mailbox)
	        gtk_signal_emit (GTK_OBJECT (tree), glacier_mbtree_signals[SELECT_MAILBOX], mailbox, row, bevent);
        }
    }	
}

static void glacier_mbtree_expand (GtkCTree * ctree, GtkCTreeNode * node, gpointer data){

    MailboxNode *mbnode;
    mbnode = (MailboxNode *)gtk_ctree_node_get_row_data (ctree, node);
    mbnode->expanded = TRUE;

}

static void glacier_mbtree_collapse (GtkCTree * ctree, GtkCTreeNode * node, gpointer data){

    MailboxNode *mbnode;
    mbnode =  (MailboxNode *)gtk_ctree_node_get_row_data (ctree, node);
    mbnode->expanded = FALSE;
}



static gboolean mailbox_is_special(gchar *path){

    if (!strcmp(path, glacier_app.inbox))
	return TRUE;

    if (!strcmp(path, glacier_app.outbox))
	return TRUE;

    if (!strcmp(path, glacier_app.trash))
	return TRUE;


    return FALSE;
}


MailboxNode *glacier_mailbox_add (gchar *name, gchar *path, MailboxNode *parent, MailboxType type){

    Mailbox *mailbox;
    MailboxNode *mbnode;

    if (!parent)	
	return NULL;

    if (type == MAILBOX_MBOX && mailbox_is_special(path)){		/* already exists */
	return NULL;
    }

    mailbox = glacier_mailbox_new (type);
    mailbox->name = g_strdup (name);
    mailbox->path = g_strdup (path);
    mailbox->type = type;

    mbnode = glacier_mailbox_node_new(mailbox);
    mbnode->parent = parent;

    if (type == MAILBOX_FOLDER) {

	gchar *tmppath;
	
        tmppath = g_strdup_printf ("%s/.expanded", path);

	if (access (tmppath, F_OK) != -1)
	    mbnode->expanded = TRUE;
        else
	    mbnode->expanded = FALSE;

	g_free(tmppath);
    }

    parent->children = g_list_insert_sorted(parent->children, mbnode, glacier_compare_mailboxes);
    
    return mbnode;
}

static GList *glacier_mailbox_find_in_list(MailboxNode *mbnode){

    gchar 	*name;
    Mailbox	*mailbox;
    GList	*mblist = NULL;

    if (mbnode->parent)
	mblist = mbnode->parent->children;
    
    if (!mblist)
	mblist = glacier_app.mailbox_list;

    mblist = g_list_first(mblist);

    name = (gchar *)((Mailbox *)(mbnode->mailbox)->name);

    while (mblist){
        mbnode = (MailboxNode *)(mblist->data);
	mailbox = (Mailbox*)(mbnode->mailbox);

	if (!strcmp(name, mailbox->name))
	    return mblist;

	mblist = g_list_next(mblist);
    }
    
    return NULL;
}	    

MailboxNode *glacier_mailbox_add_root (gchar *name, gchar *path, GList **list, MailboxType type, gboolean expanded){

    Mailbox *mailbox;
    MailboxNode *mbnode;
    GList *newlist;

    mailbox = glacier_mailbox_new (type);
    mailbox->name = g_strdup (name);
    mailbox->path = g_strdup (path);
    mailbox->type = type;

    mbnode = glacier_mailbox_node_new(mailbox);

    newlist = g_list_insert_sorted(*list, mbnode, glacier_compare_mailboxes);
    *list = newlist;
    mbnode->expanded = expanded;
    
    return mbnode;
}


static Mailbox *glacier_mailbox_new (MailboxType type) {

    Mailbox *mailbox;

    mailbox = (Mailbox *) g_malloc (sizeof (Mailbox));
    memset(mailbox, 0, sizeof(Mailbox));

    return mailbox;
}

static void glacier_mailbox_free (Mailbox *mailbox){

    if (!mailbox)
	return;

    g_free (mailbox->name);
    g_free (mailbox->path);
    g_free (mailbox);
}


static MailboxNode *glacier_mailbox_node_new (Mailbox * mailbox){

    MailboxNode *mbnode;
    mbnode = g_malloc (sizeof (MailboxNode));

    if (mbnode)
	mbnode->mailbox = mailbox;
    else
	mbnode->mailbox = NULL;

    mbnode->parent = NULL;
    mbnode->children = NULL;
    mbnode->ctree_node = NULL;

    mailbox->mbnode = mbnode;

    return mbnode;
}

static void glacier_mailbox_node_free (MailboxNode *mbnode){

    if (!mbnode)
	return;

    if (mbnode->mailbox)
	glacier_mailbox_free(mbnode->mailbox);

    g_free (mbnode);
}


static void glacier_mbtree_read_dir(gchar *path, MailboxNode *parent){

    DIR 		*dir;
    struct dirent 	*dirent;
    gchar		filename[PATH_MAX];
    struct 		stat stats;

    dir = opendir (path);

    if (!dir){
	/* ERROR */
	return;
    }

    while ((dirent = readdir (dir)) != NULL) {

	snprintf (filename, PATH_MAX, "%s/%s", path, dirent->d_name);

	if (dirent->d_name[0] == '.' || stat (filename, &stats) == -1 || access (filename, R_OK) == -1)
	    continue;

	if (S_ISDIR (stats.st_mode)) {

	    MailboxNode *mbnode = glacier_mailbox_add(dirent->d_name, filename, parent, MAILBOX_FOLDER);

	    if (mbnode)
		glacier_mbtree_read_dir(filename, mbnode);
    
        } else {

	    glacier_mailbox_add(dirent->d_name, filename, parent, MAILBOX_MBOX);

	}
    }

    closedir (dir);
}

void glacier_mbtree_load_mailboxes(void){

    gchar	*maildir;
    MailboxNode *local;

    maildir = g_strdup(glacier_app.mail_dir);

    glacier_app.mailbox_list = NULL;
    local =  glacier_mailbox_add_root (_("Local Mailboxes"), maildir, &glacier_app.mailbox_list, MAILBOX_LOCAL, TRUE);
    glacier_mailbox_add(_("Inbox"), glacier_app.inbox, local, MAILBOX_INBOX);
    glacier_mailbox_add(_("Outbox"), glacier_app.outbox, local, MAILBOX_OUTBOX);
    glacier_mailbox_add(_("Trash"), glacier_app.trash, local, MAILBOX_TRASH);

    glacier_mbtree_read_dir(maildir, local);
    g_free(maildir);

    /* redraw ctree */
    glacier_mbtree_redraw (glacier_app.mailbox_tree);
}


static gint glacier_compare_mailboxes(gconstpointer a, gconstpointer b){

    Mailbox *mailbox_a, *mailbox_b;
    
    mailbox_a = (Mailbox *)(((MailboxNode *)(a))->mailbox);
    mailbox_b = (Mailbox *)(((MailboxNode *)(b))->mailbox);
    
    if (mailbox_a->type == MAILBOX_FOLDER && mailbox_b->type != MAILBOX_FOLDER)
	return -1;
	
    if (mailbox_b->type == MAILBOX_FOLDER && mailbox_a->type != MAILBOX_FOLDER)
	return 1;
    
    if (!mailbox_b->name || !mailbox_a->name)
        return 0;

    return strcmp(mailbox_b->name, mailbox_a->name);
}

static gint glacier_mbtree_on_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer data){

    GlacierMbTree	*mbtree;
    GtkCTree 		*ctree;
    GtkCTreeNode 	*node;

    gint 		row, column;
    gint 		on_mailbox;
    Mailbox 		*mailbox;
    MailboxNode		*mbnode;

    mbtree = GLACIER_MBTREE(widget);
    ctree = GTK_CTREE (widget);
  
    on_mailbox=gtk_clist_get_selection_info (GTK_CLIST(ctree), event->x, event->y, &row, &column);
  
    if (on_mailbox){
	mbnode = gtk_clist_get_row_data (GTK_CLIST(ctree), row);

	if (mbnode) {
    	    mailbox = mbnode->mailbox;

            node = gtk_ctree_node_nth( ctree, row );
	  
    	    if (event->button == 1 && event->type == GDK_2BUTTON_PRESS){	// double click with button left 
 
    		if (mailbox->type == MAILBOX_MBOX || mailbox->type == MAILBOX_INBOX ||
		    mailbox->type == MAILBOX_OUTBOX || mailbox->type == MAILBOX_TRASH){
    		    msglist_open_mailbox(mailbox);
		}		
 
		return TRUE;
	    }
     
    	    if (event && event->button == 3){

		if (node) gtk_ctree_select(ctree, node);
	    	    gtk_menu_popup (GTK_MENU (glacier_mbtree_create_context_menu(ctree, mbnode)),  NULL, NULL, NULL, NULL, event->button, event->time);

	    }
	}

        return FALSE;

    }  else { // not on_mailbox 

        if (event->type == GDK_BUTTON_PRESS && event->button == 3){

	    // simple click on right button
	    gtk_menu_popup (GTK_MENU (glacier_mbtree_create_context_menu (ctree, NULL)), NULL, NULL, NULL, NULL, event->button, event->time); 
	    return TRUE;
	}
      
        return FALSE;
    }

    return FALSE; // never reached but this avoid compiler warning
    

}

/*************************** MENUS AND MENU CALLBACKS  ***************************/

static GtkWidget *glacier_mbtree_create_context_menu (GtkCTree * ctree, MailboxNode *mbnode){

    GtkWidget *menu, *menuitem;
    Mailbox *mailbox = mbnode->mailbox;

    menu = gtk_menu_new ();

    if (mailbox && mailbox->type == MAILBOX_MBOX) {
        menuitem = gtk_menu_item_new_with_label (_ ("Open Mailbox"));
        gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (glacier_mbtree_on_menu_open), mbnode);
        gtk_menu_append (GTK_MENU (menu), menuitem);
        gtk_widget_show (menuitem);
    }
  
    menuitem = gtk_menu_item_new_with_label (_ ("Add New Mailbox"));
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",GTK_SIGNAL_FUNC (glacier_mbtree_on_menu_add), mbnode);
    gtk_menu_append (GTK_MENU (menu), menuitem);
    gtk_widget_show (menuitem);
  
    if (mailbox && mailbox->type == MAILBOX_MBOX) {
	menuitem = gtk_menu_item_new_with_label (_ ("Edit Mailbox Properties"));
        gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (glacier_mbtree_on_menu_edit), mbnode);
        gtk_menu_append (GTK_MENU (menu), menuitem);
        gtk_widget_show (menuitem);
    }
      
    if (mailbox && (mailbox->type == MAILBOX_MBOX || mailbox->type == MAILBOX_FOLDER)) {

        menuitem = gtk_menu_item_new_with_label (_ ("Delete Mailbox"));
        gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (glacier_mbtree_on_menu_delete), mbnode);
        gtk_menu_append (GTK_MENU (menu), menuitem);
        gtk_widget_show (menuitem);
    }       

    return menu; 
}

static void glacier_mbtree_on_menu_open (GtkWidget * widget, MailboxNode * mbnode){

}

static void glacier_mbtree_on_menu_add (GtkWidget * widget, MailboxNode * mbnode){

    glacier_dialog_mailbox_add_mailbox (mbnode);
}

static void glacier_mbtree_on_menu_edit (GtkWidget * widget, MailboxNode * mbnode){

}

static void glacier_mbtree_on_menu_delete (GtkWidget * widget, MailboxNode * mbnode){

    if (mbnode)
        glacier_dialog_mailbox_conf_delete(mbnode);
}


/*************************** DIALOGS ***************************/

static void glacier_dialog_mailbox_conf_delete (MailboxNode *mbnode){

    gchar *msg;
    gint clicked_button;
    GtkWidget *ask;
    Mailbox *mailbox = mbnode->mailbox;

    if (!mailbox)
	return;
    
    if (mailbox->type == MAILBOX_MBOX){
        msg = _("The mailbox will be permanently deleted from your system.\n"
	       "Are you sure you want to remove this mailbox?");
    } else if (mailbox->type == MAILBOX_FOLDER){
        msg = _("The folder and all underlying mailboxes will be permanently\n"
	        "deleted from your system. Are you sure you want to remove this folder?");
    } else {
	return; // We wouldn't be here! What went wrong ? 
    }

    ask = gnome_message_box_new (msg, GNOME_MESSAGE_BOX_QUESTION, GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);

    gnome_dialog_set_default (GNOME_DIALOG (ask), 1);
    gtk_window_set_modal (GTK_WINDOW (ask), TRUE);
    clicked_button = gnome_dialog_run (GNOME_DIALOG (ask));

    if (clicked_button == 0){
	glacier_mailbox_delete(mbnode);
    } 
}

static void glacier_dialog_mailbox_add_mailbox (MailboxNode * mbnode){

    GtkWidget *dialog;
    GtkWidget *label1, *label2;
    GtkWidget *table1;
    GtkWidget *entry1;
    GtkWidget *checkbutton1;
    int clicked_button;

    gchar 	*mailbox_name;
    Mailbox 	*mailbox;
    MailboxNode *parent_mbnode;
    gboolean	is_folder;

    
    dialog = gnome_dialog_new("New mailbox", GNOME_STOCK_BUTTON_CANCEL, GNOME_STOCK_BUTTON_OK, NULL);    

    gtk_widget_set_usize (dialog, 300, 200);
    gtk_object_set_data( GTK_OBJECT(dialog), "dialog", (gpointer)&dialog);

    gtk_container_border_width(GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), 10);

    label1 = gtk_label_new ("Create new mailbox");
    gtk_object_set_data (GTK_OBJECT (dialog), "label1", (gpointer)&label1);
    gtk_widget_show (label1);
    gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
 
    table1 = gtk_table_new (3, 2, FALSE);
    gtk_object_set_data (GTK_OBJECT (dialog), "table1", (gpointer)&table1);
    gtk_widget_show (table1);
    gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(dialog)->vbox), table1, TRUE, TRUE, 0);
    gtk_table_set_row_spacings (GTK_TABLE (table1), 5);
    gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

    entry1 = gtk_entry_new ();
    gtk_object_set_data (GTK_OBJECT (dialog), "entry1", (gpointer)&entry1);
    gtk_widget_show (entry1);
    gtk_table_attach (GTK_TABLE (table1), entry1, 1, 2, 1, 2, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

    checkbutton1 = gtk_check_button_new_with_label ("Make it folder");
    gtk_object_set_data (GTK_OBJECT (dialog), "checkbutton1", (gpointer)&checkbutton1);
    gtk_widget_show (checkbutton1);
    gtk_table_attach (GTK_TABLE (table1), checkbutton1, 1, 2, 2, 3, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

    label2 = gtk_label_new ("Mailbox name:");
    gtk_object_set_data (GTK_OBJECT (dialog), "label2", (gpointer)&label2);
    gtk_widget_show (label2);
    gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 1, 2, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
    gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
    gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    clicked_button = gnome_dialog_run (GNOME_DIALOG (dialog));

    mailbox_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry1)));
    is_folder = (gboolean)gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton1));

    gtk_widget_destroy (dialog);

    if (clicked_button == 1){     

    	mailbox = mbnode->mailbox;

	if (mailbox->type == MAILBOX_FOLDER || mailbox->type == MAILBOX_LOCAL){
	    parent_mbnode = mbnode;

	} else if (mailbox->type == MAILBOX_MBOX){
	    parent_mbnode = mbnode->parent;

	} else {
	    parent_mbnode = NULL;
	}

	if (parent_mbnode)
	    glacier_mailbox_create(parent_mbnode, mailbox_name, is_folder);

	g_free(mailbox_name);
    }
}

static void glacier_mailbox_create(MailboxNode *parent_mbnode, gchar *name, gint folder){

    gchar *path;
    MailboxType type;
    gboolean created;

    if (!name || *name == '\0')
	return;

    if (!parent_mbnode){
        path = g_strdup_printf ("%s/%s", glacier_app.mail_dir, name);
    } else {
        path = g_strdup_printf ("%s/%s", parent_mbnode->mailbox->path, name);
    }

    if (folder){
	type = MAILBOX_FOLDER;
	created = glacier_utils_create_dir(path, 0700);
    } else {
	type = MAILBOX_MBOX;
	created = glacier_utils_create_file(path, S_IRUSR | S_IWUSR);
    }

    if (created){
        glacier_mailbox_add(name, path, parent_mbnode, type);
	glacier_mbtree_redraw(glacier_app.mailbox_tree);
    }

    g_free(path);
}


static void glacier_mailbox_delete_list(GList *list){

    MailboxNode *mbnode;
    GList *glist;

    glist = g_list_first(list);      
    
    while (glist){
	mbnode = glist->data;
	if (mbnode->children)
	    glacier_mailbox_delete_list(mbnode->children);

	glacier_mailbox_node_free(mbnode);
	glist = g_list_next(glist);
    }

    g_list_free(list);
}


static void glacier_mailbox_delete(MailboxNode *mbnode){

    GList *glist;

    glist = glacier_mailbox_find_in_list(mbnode);
    
    if (!glist){
	fprintf (stderr, _("Oooops! mailbox not found!\n"));

    } else {

	if (mbnode->children)
	    glacier_mailbox_delete_list(mbnode->children);
	    
	if (glist->prev == NULL){
	
	    if (mbnode->parent)
		mbnode->parent->children = g_list_next(glist);
	    else     
		glacier_app.mailbox_list = g_list_next(glist);;
	
	}
	g_list_remove(glist, mbnode);
	glacier_utils_remove_files(mbnode->mailbox->path);
	glacier_mailbox_node_free (mbnode);	
	
	glacier_mbtree_redraw(glacier_app.mailbox_tree);
    } 
}	

void glacier_mbtree_close_mailboxes(GList *list){

    GList	*mblist;
    MailboxNode *mbnode;
    Mailbox	*mailbox;
    gchar	*tmpfile;

    mblist = g_list_first(list);
    mbnode = (MailboxNode *)((MailboxNode *)(mblist->data))->parent;

    while (mblist){
        mbnode = (MailboxNode *)(mblist->data);
	mailbox = (Mailbox*)(mbnode->mailbox);
	    
	switch (mbnode->mailbox->type){
	    
	    case MAILBOX_MBOX:
	    case MAILBOX_INBOX:
	    case MAILBOX_OUTBOX:
	    case MAILBOX_TRASH:
		break;

	    case MAILBOX_FOLDER:
	        if (mbnode->expanded){
		    tmpfile = g_strdup_printf ("%s/.expanded", mbnode->mailbox->path);
		    glacier_utils_create_file(tmpfile, S_IRUSR | S_IWUSR);
		    g_free (tmpfile);
		} else {
		    tmpfile = g_strdup_printf ("%s/.expanded", mbnode->mailbox->path);
		    if (access (tmpfile, F_OK) != -1)
			unlink (tmpfile);
		    g_free (tmpfile);
		}
		break;

	    default:
		break;
	}

	if (mbnode->children)
	    glacier_mbtree_close_mailboxes(mbnode->children);

	mblist = g_list_next(mblist);
    }
}

void glacier_utils_remove_files (gchar *path){

    gchar cmd[PATH_MAX + 12];
    snprintf (cmd, sizeof (cmd), "rm -rf '%s'", path);
    system (cmd);
}

gboolean glacier_utils_create_dir (gchar *path, int mode){

   if (mkdir(path, mode) == -1){
	GtkWidget *msgbox;
	gchar *ptr;
	ptr = g_strdup_printf (_("Cannot create directory '%s'\nError: %s\n"), path, strerror (errno));
	msgbox = gnome_message_box_new (ptr, GNOME_MESSAGE_BOX_ERROR, _("OK"), NULL);
	free (ptr);
	gtk_window_set_modal (GTK_WINDOW (msgbox), TRUE);
	gnome_dialog_run (GNOME_DIALOG (msgbox));
	return FALSE;
    }

    return TRUE;
}

gboolean glacier_utils_create_file(gchar *filename, int mode){

    int fd = creat (filename, mode);

    if (fd < 0) {
	GtkWidget *msgbox;
	gchar *ptr;
	ptr = g_strdup_printf (_("Cannot create file '%s'\nError: %s\n"), filename, strerror (errno));
	msgbox = gnome_message_box_new (ptr, GNOME_MESSAGE_BOX_ERROR, _("OK"), NULL);
	free (ptr);
	gtk_window_set_modal (GTK_WINDOW (msgbox), TRUE);
	gnome_dialog_run (GNOME_DIALOG (msgbox));
	return FALSE;
    }
    
    close (fd);
    return TRUE;
}

