/*
 * Glacier - another Gnome email client.
 * Copyright (C) 1999 Lubomir Gelo
 *
 * Glacier uses code from other GPL-ed projects. See THANKS for 
 * credits and copyright info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "includes.h"

static void on_row_selected(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
static void on_click_column (GtkCList * list, gint column, gpointer data);

static gint msglist_compare_messages_by_date(GtkCList *list, gconstpointer a, gconstpointer b);
static gint msglist_compare_messages_by_subject(GtkCList *list, gconstpointer a, gconstpointer b);
static gint msglist_compare_messages_by_sender(GtkCList *list, gconstpointer a, gconstpointer b);

static void msglist_set_row_imagees(GtkCList *list, gint row, Message *message);

GtkWidget *glacier_create_message_list(GtkWidget *paned, int position) {

    gchar *titles[6] = { "S", "D", "Date", "Time", "From", "Subject"};

    GtkWidget *scrolled, *list;

    /* Create a scrolled window to hold the list. */
    scrolled = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);    

    gtk_container_set_border_width (GTK_CONTAINER (scrolled), 5);

    list =  gtk_clist_new_with_titles (6, titles);

    gtk_container_add(GTK_CONTAINER(scrolled), list);

    gtk_signal_connect(GTK_OBJECT(list), "select_row", GTK_SIGNAL_FUNC(on_row_selected), NULL);
    gtk_signal_connect (GTK_OBJECT (list), "click_column", GTK_SIGNAL_FUNC (on_click_column), NULL);

    gtk_clist_set_shadow_type (GTK_CLIST (list), GTK_SHADOW_IN);


    gtk_clist_set_column_justification (GTK_CLIST(list), 0, GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_justification (GTK_CLIST(list), 1, GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_width (GTK_CLIST(list), 0, 16);
    gtk_clist_set_column_width (GTK_CLIST(list), 1, 16);
    gtk_clist_set_column_width (GTK_CLIST(list), 2, 60);
    gtk_clist_set_column_width (GTK_CLIST(list), 3, 40);
    gtk_clist_set_column_width (GTK_CLIST(list), 4, 100);
    gtk_clist_set_column_width (GTK_CLIST(list), 5, 120);

    gtk_clist_set_selection_mode (GTK_CLIST (list), GTK_SELECTION_EXTENDED);
    gtk_clist_column_titles_active (GTK_CLIST (list));
    gtk_clist_set_reorderable(GTK_CLIST (list), TRUE);

    gtk_widget_set_usize(scrolled, glacier_app.msglist_width, glacier_app.msglist_height);	

    if (position == 1)
	gtk_paned_add1 (GTK_PANED(paned), scrolled);
    else
	gtk_paned_add2 	(GTK_PANED(paned), scrolled);


    gtk_widget_show (list);
    gtk_widget_show (scrolled);
    
    return list;
}

void msglist_open_mailbox(Mailbox *mbox){

    char 	buff[PATH_MAX + 20];
    int 	count;
    GList	*cur_msg;
    Message	*message;
    gchar	*fields[6], *string_date, *string_time;
    gint	row;

    if (mbox->type == MAILBOX_FOLDER || mbox->type == MAILBOX_LOCAL)
	return;

    set_cursor(GDK_WATCH);

    snprintf (buff, PATH_MAX + 20, "Glacier - %s",  mbox->path);
    gtk_window_set_title(GTK_WINDOW (glacier_app.app), buff);

    if (glacier_app.current_mailbox){
	mbox_close_mailbox(glacier_app.current_mailbox);
	glacier_app.current_mailbox = NULL;
    }
    
    /* clear list */
    gtk_clist_clear(GTK_CLIST (glacier_app.message_list));

    count = mbox_open_mailbox(mbox, TRUE);

    if (count > 0){
	glacier_app.current_mailbox = mbox;

	/* load messages */
    	gtk_clist_freeze(GTK_CLIST (glacier_app.message_list));

	cur_msg = g_list_first(mbox->messages);
	
	while (cur_msg){

	    message = (Message *)(cur_msg->data);
	    utils_print_friendly_time(message->header->date_sent, &string_date, &string_time);
	    fields[0] = NULL;
	    fields[1] = NULL;
	    fields[2] = string_date;
	    fields[3] = string_time;
	    fields[4] = address_print_short(message->envelope->from, 0);
	    fields[5] = message->envelope->subject;

    	    row = gtk_clist_append (GTK_CLIST(glacier_app.message_list), fields);	    
	    gtk_clist_set_row_data (GTK_CLIST(glacier_app.message_list), row, (gpointer) message);
	    msglist_set_row_imagees(GTK_CLIST(glacier_app.message_list), row, message);

	    G_FREE(fields[2]);
	    G_FREE(fields[3]);
	    G_FREE(fields[4]);

	    cur_msg = g_list_next(cur_msg);
	}


	GTK_CLIST (glacier_app.message_list)->sort_type = GTK_SORT_ASCENDING;
	gtk_clist_set_sort_column (GTK_CLIST(glacier_app.message_list), 2);
	gtk_clist_set_compare_func (GTK_CLIST(glacier_app.message_list), msglist_compare_messages_by_date);
	
	gtk_clist_sort (GTK_CLIST(glacier_app.message_list));
	gtk_clist_set_auto_sort (GTK_CLIST(glacier_app.message_list), TRUE);

	gtk_clist_thaw (GTK_CLIST(glacier_app.message_list));
    }	

    set_cursor(-1);

}

static void on_row_selected(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data){

    Message *message;
    
    message = gtk_clist_get_row_data(GTK_CLIST(glacier_app.message_list), row);
    msgview_set(glacier_app.message_preview, message, MSGVIEW_NORMAL_HEADERS);
}

static gint msglist_compare_messages_by_date(GtkCList *list, gconstpointer a, gconstpointer b){
    
    Message 	*msg_a, *msg_b;
    long 	date_a, date_b;
    
    msg_a = (Message *)(((GtkCListRow *)a)->data);
    msg_b = (Message *)(((GtkCListRow *)b)->data);

    if (!msg_a || !msg_b)
	return 0;

    date_a = msg_a->header->date_sent;
    date_b = msg_b->header->date_sent;
    
    if (date_a > date_b)
	return 1;
	
    if (date_a < date_b)
	return -1;

    return 0;
}

static gint msglist_compare_messages_by_subject(GtkCList *list, gconstpointer a, gconstpointer b){

    Message 	*msg_a, *msg_b;
    
    msg_a = (Message *)(((GtkCListRow *)a)->data);
    msg_b = (Message *)(((GtkCListRow *)b)->data);

    if (!msg_a || !msg_b || !MSG_ENVELOPE(msg_a, subject) || !MSG_ENVELOPE(msg_b, subject))
	return 0;

    return (strcmp(MSG_ENVELOPE(msg_a, subject), MSG_ENVELOPE(msg_b, subject)));
}

static gint msglist_compare_messages_by_sender(GtkCList *list, gconstpointer a, gconstpointer b){

    Message 	*msg_a, *msg_b;
    gchar	*addr_a, *addr_b;    
    gint	result;

    msg_a = (Message *)(((GtkCListRow *)a)->data);
    msg_b = (Message *)(((GtkCListRow *)b)->data);

    if (!msg_a || !msg_b)
	return 0;

    addr_a = address_print_short(MSG_ENVELOPE(msg_a, from), 0);
    addr_b = address_print_short(MSG_ENVELOPE(msg_b, from), 0);

    if (!addr_a || !addr_b){
	G_FREE(addr_a);
	G_FREE(addr_b);
	return 0;
    }

    result = strcmp(addr_a, addr_b);
    
    G_FREE(addr_a);
    G_FREE(addr_b);
    
    return result;
    
}

void msglist_select_next(GtkCList *clist){

    GList *sel;
    gint i = 0;
    gint h = 0;

   g_return_if_fail (clist != NULL);

    if (!clist->selection)
	return;

    sel = clist->selection;

    while (sel){
        i = GPOINTER_TO_INT (sel->data);
        if (i > h)
	    h = i;
        sel = g_list_next(sel);
    }

    if (h + 1 >= clist->rows)
	return;

    gtk_clist_unselect_all (clist);

    gtk_clist_select_row (clist, h + 1, -1);

    if (gtk_clist_row_is_visible (clist, h + 1) != GTK_VISIBILITY_FULL)
	gtk_clist_moveto (clist, h + 1, 0, 1.0, 0.0);
}


void msglist_select_prev(GtkCList *clist){

    GList *sel;
    gint i = 0;
    gint h = 0;

    g_return_if_fail (clist != NULL);

    if (!clist->selection)
	return;

    h = clist->rows;		/* set this to the max number of rows */

    sel = clist->selection;
    while (sel){			/* look for the selected row with the lowest number */
        i = GPOINTER_TO_INT (sel->data);
        if (i < h)
 	    h = i;
	sel = g_list_next(sel);    
    }

    /* avoid unselecting everything, and then not selecting a valid row */
    if (h < 1)
	h = 1;

    /* FIXME, if it is already on row 1, we shouldn't unselect all/reselect */
    gtk_clist_unselect_all (clist);

    gtk_clist_select_row (clist, h - 1, -1);

    if (gtk_clist_row_is_visible (clist, h - 1) != GTK_VISIBILITY_FULL)
	gtk_clist_moveto (clist, h - 1, 0, 0.0, 0.0);
}


static void on_click_column (GtkCList *list, gint column, gpointer data){

    if (column == 3)
	column = 2;

    if (column == list->sort_column){
        if (list->sort_type == GTK_SORT_ASCENDING)
	    list->sort_type = GTK_SORT_DESCENDING;
        else
	    list->sort_type = GTK_SORT_ASCENDING;
    } else
	gtk_clist_set_sort_column (list, column);

    if (column == 2)
	gtk_clist_set_compare_func (list, msglist_compare_messages_by_date);
    else if (column == 4)
	gtk_clist_set_compare_func (list, msglist_compare_messages_by_sender);
    else if (column == 5)
	gtk_clist_set_compare_func (list, msglist_compare_messages_by_subject);
    else
	gtk_clist_set_compare_func (list, NULL);

    gtk_clist_sort (list);
}

static void msglist_set_row_imagees(GtkCList *list, gint row, Message *message){

    if (message->header->deleted)
        gtk_clist_set_pixmap(list, row, 1, icon_get_pixmap (ICON_TRASH), icon_get_bitmap (ICON_TRASH));

    else if (message->header->replied)
	gtk_clist_set_pixmap(list, row, 1, icon_get_pixmap (ICON_REPLIED), icon_get_bitmap (ICON_REPLIED));

    else if (! message->header->read)
	gtk_clist_set_pixmap (list, row, 1, icon_get_pixmap (ICON_ENVELOPE), icon_get_bitmap (ICON_ENVELOPE));
    else
	gtk_clist_set_text (list, row, 1, NULL);
}


