#include "includes.h"

#define BGLINKCOLOR 		"LightSteelBlue1"

typedef struct _SaveFileInfo SaveFileInfo;

struct _SaveFileInfo {

    GtkWidget 	*file_entry;
    Message 	*msg;
    Body 	*body;
};


SaveFileInfo *save_file_info_new (GtkWidget * widget, Message * message, Body * body);
static void msgview_item_event (GnomeCanvasItem * item, GdkEvent * event, gpointer data);
static void save_mime_part (GtkObject * o, SaveFileInfo *);

/* widget */
static void msgview_class_init (GlacierMsgviewClass * klass);
static void msgview_init (GlacierMsgview * msgview);
static void msgview_size_request (GtkWidget * widget, GtkRequisition * requisition);
static void msgview_size_allocate (GtkWidget * widget, GtkAllocation * allocation);

static void msgview_raw2canvas (GlacierMsgview *msgview, Message * message);
static void msgview_headers2canvas (GlacierMsgview *msgview, Message * message);
static void msgview_body2canvas (GlacierMsgview *msgview, Message * message);
static gboolean msgview_content2canvas (Message * message, GnomeCanvasGroup * group);

static void msgview_part2canvas (Message * message, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_other2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_mimetext2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_video2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_multipart2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_message2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_image2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_application2canvas (Message * message, Body * body, FILE * fp, GnomeCanvasGroup * group);
static void msgview_audio2canvas (Message *, Body * body, FILE * fp, GnomeCanvasGroup * group);

static gchar *save_message_part (Message * message, Body * body);

static gint msgview_key_pressed (GtkWidget *widget, GdkEventKey *event, gpointer callback_data);
static void msgview_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer callback_data);

static GnomeCanvasClass *parent_class = NULL;

guint msgview_get_type(){

    static guint msgview_type = 0;

    if (!msgview_type) {
      
          GtkTypeInfo msgview_info = { 	"GlacierMsgview",
					sizeof (GlacierMsgview),
					sizeof (GlacierMsgviewClass),
		    			(GtkClassInitFunc) msgview_class_init,
					(GtkObjectInitFunc) msgview_init,
					(GtkArgSetFunc) NULL,
					(GtkArgGetFunc) NULL,
					(GtkClassInitFunc) NULL};

          msgview_type = gtk_type_unique (gnome_canvas_get_type(), &msgview_info);
    }

    return msgview_type;
}

static void msgview_class_init(GlacierMsgviewClass * klass){

    GtkWidgetClass *widget_class;

    widget_class = (GtkWidgetClass *)klass;

    widget_class->size_request = msgview_size_request;
    widget_class->size_allocate = msgview_size_allocate;

    parent_class = gtk_type_class(gnome_canvas_get_type());
}

static void msgview_init(GlacierMsgview * msgview){

    msgview->message = NULL;
    msgview->headers = NULL;
    msgview->body = NULL;
    msgview->message = NULL;
}

static void msgview_item_event(GnomeCanvasItem * item, GdkEvent * event, gpointer data){

    SaveFileInfo 		*info;

    GtkWidget 			*save_dialog;
    GtkWidget 			*file_entry;
  
    GdkCursor 			*cursor = NULL;

    info = data;

    switch (event->type) {
    
	case GDK_BUTTON_PRESS:

            save_dialog = gnome_dialog_new ( _("Save Attachment"), _("Save"), _("Cancel"), NULL);
            file_entry = gnome_file_entry_new ("Glacier_MIME_Saver", _("Save MIME Part"));
            info->file_entry = file_entry;

            if (info->body->filename)	{
		gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry))), info->body->filename);
	    }

            gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (save_dialog)->vbox), file_entry, FALSE, FALSE, 10);
            gtk_widget_show (file_entry);
            gnome_dialog_button_connect (GNOME_DIALOG (save_dialog), 0, save_mime_part, info);
            gtk_window_set_modal (GTK_WINDOW (save_dialog), TRUE);
            gnome_dialog_run (GNOME_DIALOG (save_dialog));
            gtk_widget_destroy (save_dialog);
            save_dialog = NULL;
            break;

	case GDK_ENTER_NOTIFY:
            cursor = gdk_cursor_new (GDK_HAND2);
            gdk_window_set_cursor (GTK_LAYOUT (item->canvas)->bin_window, cursor);
            gdk_cursor_destroy (cursor);

            if (!GNOME_IS_CANVAS_IMAGE (item))
                gnome_canvas_item_set (item, "fill_color", "red", NULL);

            break;

	case GDK_LEAVE_NOTIFY:
            cursor = gdk_cursor_new (GDK_HAND1);
            gdk_window_set_cursor (GTK_LAYOUT (item->canvas)->bin_window, NULL);

            if (!GNOME_IS_CANVAS_IMAGE (item))
                gnome_canvas_item_set (item, "fill_color", "black", NULL);

            break;

	default:
            break;
    }
}


GlacierMsgview *msgview_new(void){

    GlacierMsgview *msgview;
    GtkStyle *style;
    GdkColormap* colormap;
  
    gtk_widget_push_visual(gdk_imlib_get_visual());
    gtk_widget_push_colormap(gdk_imlib_get_colormap());
    msgview = gtk_type_new(msgview_get_type());
    gtk_widget_pop_visual();
    gtk_widget_pop_colormap();

    style =  gtk_style_new ();
    colormap = gtk_widget_get_colormap(GTK_WIDGET(msgview));
    gdk_color_white(colormap, &style->bg[GTK_STATE_NORMAL]);
    gtk_widget_set_style(GTK_WIDGET(msgview), style);

    gtk_signal_connect_after(GTK_OBJECT(msgview), "key_press_event", GTK_SIGNAL_FUNC(msgview_key_pressed), NULL);
    gtk_signal_connect(GTK_OBJECT(msgview), "button_press_event", GTK_SIGNAL_FUNC(msgview_button_pressed), NULL);
 
    return msgview;
}

static void msgview_size_request(GtkWidget * widget, GtkRequisition * requisition){

    g_return_if_fail (widget != NULL);
    g_return_if_fail (GLACIER_IS_MSGVIEW (widget));
    g_return_if_fail (requisition != NULL);

    if (GTK_WIDGET_CLASS (parent_class)->size_request)
	(*GTK_WIDGET_CLASS (parent_class)->size_request)(widget, requisition );

    requisition->width = 100;
    requisition->height = 100;
}

static void msgview_size_allocate (GtkWidget * widget, GtkAllocation * allocation){

    double x1, x2, y1, y2;
    g_return_if_fail (widget != NULL);
    g_return_if_fail (GLACIER_IS_MSGVIEW (widget));
    g_return_if_fail (allocation != NULL);

    if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
	(*GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);


    gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (GNOME_CANVAS_GROUP (
				  GNOME_CANVAS (widget)->root)),
				  &x1, &y1, &x2, &y2);

    gnome_canvas_set_scroll_region(	GNOME_CANVAS (widget),
				        0, 0,
			                (x2 > allocation->width) ? x2 : allocation->width,
		                        (y2 > allocation->height) ? y2 : allocation->height);

}

gint msgview_key_pressed (GtkWidget *widget, GdkEventKey *event, gpointer callback_data){

    int 	x, y;
    double 	x2,x3, y2,y3;

    if (event->keyval == GDK_Up) {
        gnome_canvas_get_scroll_offsets(GNOME_CANVAS(widget), &x, &y);
        gnome_canvas_scroll_to(GNOME_CANVAS(widget), x, y-15);
        return (TRUE);
    }

    if (event->keyval == GDK_Down) {
        gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
	gnome_canvas_scroll_to (GNOME_CANVAS(widget), x, y+15);
        return (TRUE);
    }

    if (event->keyval == GDK_Left) {
         gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
         gnome_canvas_scroll_to (GNOME_CANVAS(widget), x-15, y);
         return (TRUE);
    }

    if (event->keyval == GDK_Right) {
	gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
        gnome_canvas_scroll_to (GNOME_CANVAS(widget), x+15, y);
        return (TRUE);
    }

    if (event->keyval == GDK_Page_Up) {
	gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
        gnome_canvas_scroll_to (GNOME_CANVAS(widget), x, y-(GTK_WIDGET (widget))->allocation.height);
      return (TRUE);
    }

    if (event->keyval == GDK_Page_Down) {
        gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
        gnome_canvas_scroll_to (GNOME_CANVAS(widget), x, y+(GTK_WIDGET (widget))->allocation.height);
        return (TRUE);
    }

    if (event->keyval == GDK_Home) {
        gnome_canvas_get_scroll_region (GNOME_CANVAS(widget), &x2, &y2, &x3, &y3);
        gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
        gnome_canvas_scroll_to (GNOME_CANVAS(widget), x, 0);
        return (FALSE);
    }

    if (event->keyval == GDK_End) {
        gnome_canvas_get_scroll_region (GNOME_CANVAS(widget), &x2, &y2, &x3, &y3);
        gnome_canvas_get_scroll_offsets (GNOME_CANVAS(widget), &x, &y);
        gnome_canvas_scroll_to (GNOME_CANVAS(widget), x, y3);
        return (FALSE);
    }

    return (FALSE);
}

void msgview_button_pressed (GtkWidget *widget, GdkEventButton *event, gpointer callback_data){

    (*(GTK_WIDGET_CLASS (GTK_WIDGET(widget)->object.klass)->grab_focus))(widget);

}

void msgview_set (GlacierMsgview *msgview, Message *message, int mode){

    GtkAllocation *alloc = NULL;
    GnomeCanvasGroup *bm_group;
    double x1, x2, y1, y2;

    g_return_if_fail(msgview != NULL);
    g_return_if_fail(msgview != NULL);

    if (msgview->message == message)
	return;

    if (msgview->headers) {
	gtk_object_destroy(GTK_OBJECT(msgview->headers));
        gtk_object_destroy(GTK_OBJECT(msgview->body));
    }

    gnome_canvas_scroll_to(GNOME_CANVAS(msgview), 0, 0);

    if (mode == MSGVIEW_RAW_MESSAGE){
	msgview_raw2canvas(msgview, message);
    } else {
	message_parse_mime_message(message);
        msgview_headers2canvas(msgview, message);
        msgview_body2canvas(msgview, message);
	bm_group = GNOME_CANVAS_GROUP(GNOME_CANVAS(msgview)->root);

        gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM(bm_group), &x1, &y1, &x2, &y2);

        alloc = &(GTK_WIDGET(msgview)->allocation);
	gnome_canvas_set_scroll_region(GNOME_CANVAS(msgview), 0, 0, (x2 > alloc->width) ? x2 : alloc->width, (y2 > alloc->height) ? y2 : alloc->height);
    }
}

static GnomeCanvasItem *msgview_text_item(gchar * text, GnomeCanvasGroup * group, double x, double y){

    GnomeCanvasItem *new;

    new = gnome_canvas_item_new(group, GNOME_TYPE_CANVAS_TEXT,
				"x", x,
			        "y", y,
			        "anchor", GTK_ANCHOR_NW,
				"font", "-adobe-helvetica-medium-r-normal--12-*-72-72-p-*-iso8859-1",
			        "text", text, NULL);
    return new;
}

static GnomeCanvasItem *msgview_text_item_set_bg(GnomeCanvasItem * item, GnomeCanvasGroup * group, gchar * color){

    double x1, x2, y1, y2;
    GnomeCanvasItem *new;

    gnome_canvas_item_get_bounds(item, &x1, &y1, &x2, &y2);

    new = gnome_canvas_item_new(group, gnome_canvas_rect_get_type(),
				"x1", x1,
			        "y1", y1,
			        "x2", x2,
			        "y2", y2,
			        "fill_color", color,
			        NULL);

    gnome_canvas_item_lower (new, 1);

    return new;
}

static double msgview_next_part_height(GnomeCanvasGroup * group){

    double x1, x2, y1, y2, r;

    gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM (group), &x1, &y1, &x2, &y2);

    r = y2 - y1;
    r += 25;

    return r;
}

static double msgview_next_row_height(GnomeCanvasGroup * row[]){

    double o, t, x1, x2, y1, y2;

    gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (row[0]), &x1, &y1, &x2, &y2);
    o = y2 - y1;

    gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (row[1]), &x1, &y1, &x2, &y2);
    t = y2 - y1;

    if (o > t)
	return o;
    else
	return t;
}

static void msgview_headers2canvas(GlacierMsgview *msgview, Message *message){

    double 		next_height = 0;
    double 		x1, x2, y1, y2;

    GnomeCanvasGroup 	*m_root;
    GnomeCanvasGroup 	*row[2];
    gchar		*buf;

    m_root = GNOME_CANVAS_GROUP(GNOME_CANVAS(msgview)->root);

    msgview->headers = GNOME_CANVAS_GROUP(gnome_canvas_item_new(m_root, GNOME_TYPE_CANVAS_GROUP, "x", (double) 10.0, 
								"y", (double) 10.0, NULL));

    row[0] = GNOME_CANVAS_GROUP(gnome_canvas_item_new(msgview->headers, GNOME_TYPE_CANVAS_GROUP, "x", (double) 0.0,
						      "y", (double) 0.0, NULL));

    row[1] = GNOME_CANVAS_GROUP(gnome_canvas_item_new(msgview->headers, GNOME_TYPE_CANVAS_GROUP, "x", (double) 0.0,
						      "y", (double) 0.0, NULL));

    msgview_text_item (_("Date:"), row[0], 0.0, 0.0);
    buf = utils_print_time(message->header->date_sent);
    if (buf){
        msgview_text_item (buf, row[1], 0.0, 0.0);
        g_free(buf);
    }

    if (MSG_ENVELOPE(message, to)){
        next_height = msgview_next_row_height (row);
	msgview_text_item (_("To:"), row[0], 0.0, next_height);
        buf = address_list_print(MSG_ENVELOPE(message, to));
	if (buf){
    	    msgview_text_item (buf, row[1], 0.0, next_height);
	    g_free(buf);
        }
    }	

    if (MSG_ENVELOPE(message, cc)){
        next_height = msgview_next_row_height (row);
	msgview_text_item (_("Cc:"), row[0], 0.0, next_height);
        buf = address_list_print(MSG_ENVELOPE(message, cc));
	if (buf){
    	    msgview_text_item (buf, row[1], 0.0, next_height);
    	    g_free(buf);
	}
    }	

    if (MSG_ENVELOPE(message, from)){
        next_height = msgview_next_row_height (row);
	msgview_text_item (_("From:"), row[0], 0.0, next_height);
        buf = address_list_print(MSG_ENVELOPE(message, from));
	if (buf){
    	    msgview_text_item (buf, row[1], 0.0, next_height);
    	    g_free(buf);
	}
    }	

    if (MSG_ENVELOPE(message, subject)){
        next_height = msgview_next_row_height (row);
	msgview_text_item (_("Subject:"), row[0], 0.0, next_height);
        msgview_text_item (MSG_ENVELOPE(message,subject), row[1], 0.0, next_height);
    }	

    gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (row[0]), &x1, &y1, &x2, &y2);
    gnome_canvas_item_move (GNOME_CANVAS_ITEM (row[1]), x2 - x1 + 50, 0.0);
}


static void msgview_body2canvas (GlacierMsgview *msgview, Message *message){

    GnomeCanvasGroup *m_root;
    double x1, x2, y1, y2;

    m_root = GNOME_CANVAS_GROUP(GNOME_CANVAS(msgview)->root);

    gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(msgview->headers), &x1, &y1, &x2, &y2);

    msgview->body = GNOME_CANVAS_GROUP(gnome_canvas_item_new(	m_root, GNOME_TYPE_CANVAS_GROUP,
					                	"x", (double) 10.0,
					                        "y", (double) (y2 - y1) + 15,
					                        NULL));

    msgview_content2canvas(message, msgview->body);
}

static void msgview_other2canvas (Message * message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    State 	s;
    gchar 	*text;
    char	*filename;
    size_t 	alloced;

    fseek (fp, body->offset, 0);
    s.fpin = fp;
    s.fpout = utils_mktemp(&filename);
    s.prefix = NULL;

    message_decode_attachment (body, &s);

    fflush (s.fpout);
    alloced = utils_readfile (s.fpout, &text);

    if (text)
	text[alloced - 1] = '\0';

    msgview_text_item (text, group, 0.0, msgview_next_part_height (group));

    g_free (text);
    fclose (s.fpout);

    unlink (filename);
    free(filename);

}

static void msgview_raw2canvas(GlacierMsgview *msgview, Message *message){

    GnomeCanvasGroup 	*m_root;
    gchar 		*text; 

    m_root = GNOME_CANVAS_GROUP(GNOME_CANVAS(msgview)->root);

    msgview->headers = GNOME_CANVAS_GROUP(gnome_canvas_item_new(m_root, GNOME_TYPE_CANVAS_GROUP, "x", (double) 10.0, 
								"y", (double) 10.0, NULL));

    text = mbox_get_raw_message(message);

    if (text){
	msgview_text_item (text, msgview->headers, 0.0, msgview_next_part_height (msgview->headers));
	g_free (text);
    }	
}


static void msgview_audio2canvas (Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    GnomeCanvasItem 	*item;
    SaveFileInfo 	*info;

    item = msgview_text_item("--AUDIO--", group, 0.0, msgview_next_part_height (group));
    msgview_text_item_set_bg(item, group, BGLINKCOLOR);

    info = save_file_info_new (NULL, message, body);
    gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (msgview_item_event), info);
}


static void msgview_application2canvas (Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    GnomeCanvasItem 	*item;
    SaveFileInfo 	*info;

    item = msgview_text_item ("--APPLICATION--", group, 0.0, msgview_next_part_height (group));
    msgview_text_item_set_bg (item, group, BGLINKCOLOR);

    info = save_file_info_new (NULL, message, body);
    gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (msgview_item_event), info);
}

static void msgview_image2canvas(Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    GnomeCanvasItem 	*item;
    SaveFileInfo 	*info;
    GdkImlibImage 	*im;
    gchar 		*filename;

    filename = save_message_part (message, body);
    im = gdk_imlib_load_image (filename);
    item = gnome_canvas_item_new (group, gnome_canvas_image_get_type (),
				"image", im,
				"x", 0.0,
				"y", msgview_next_part_height (group) + 5.0,
				"width", (double) im->rgb_width,
				"height", (double) im->rgb_height + 10.0,
				"anchor", GTK_ANCHOR_NW,
				NULL);
    info = save_file_info_new (NULL, message, body);
    gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (msgview_item_event), info);

    unlink(filename);
    free(filename);
}

static void msgview_message2canvas (Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    GnomeCanvasItem *item;
    SaveFileInfo *info;

    item = msgview_text_item ("--MESSAGE--", group, 0.0, msgview_next_part_height (group));
    msgview_text_item_set_bg (item, group, BGLINKCOLOR);

    info = save_file_info_new (NULL, message, body);
    gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (msgview_item_event), info);
}

static void msgview_multipart2canvas (Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    GList *p = body->parts;
    
    while (p) {
        msgview_part2canvas (message, p->data, fp, group);
	p = g_list_next(p);
    }
}


static void msgview_video2canvas (Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    GnomeCanvasItem 	*item;
    SaveFileInfo 	*info;

    item = msgview_text_item ("--VIDEO--", group, 0.0, msgview_next_part_height (group));
    msgview_text_item_set_bg (item, group, BGLINKCOLOR);

    info =save_file_info_new (NULL, message, body);
    gtk_signal_connect (GTK_OBJECT (item), "event", GTK_SIGNAL_FUNC (msgview_item_event), info);
}

static void msgview_mimetext2canvas (Message *message, Body *body, FILE *fp, GnomeCanvasGroup *group){

    State 	s;
    gchar 	*ptr = 0;
    size_t 	alloced;
    char	*filename;

    fseek (fp, body->offset, 0);
    s.fpin = fp;
    s.prefix = 0;
    s.fpout= utils_mktemp(&filename);
    message_decode_attachment (body, &s);

    fflush (s.fpout);
    alloced = utils_readfile (s.fpout, &ptr);

    if (ptr) {
        ptr[alloced - 1] = '\0';

        if (strcmp (body->subtype, "html") == 0)	{
  	    GnomeCanvasItem *item;
	    item =  msgview_text_item ("--HTML--", group, 0.0, msgview_next_part_height (group));
	    msgview_text_item_set_bg (item, group, BGLINKCOLOR);
	    goto END;
	}

        /* add the text item to the canvas */
        msgview_text_item (ptr, group, 0.0, msgview_next_part_height (group));
        g_free (ptr);
    }

END:
    fclose (s.fpout);
    unlink (filename);
    free (filename);

}


static void msgview_part2canvas (Message * message, Body * body, FILE * fp, GnomeCanvasGroup * group){

    switch (body->type){

	case TYPEOTHER:
	    dprint(10, (stderr, "msgview_part2canvas(): other\n"));
            msgview_other2canvas (message, body, fp, group);
            break;

	case TYPEAUDIO:
	    dprint(10, (stderr, "msgview_part2canvas(): audio\n"));
            msgview_audio2canvas (message, body, fp, group);
            break;

	case TYPEAPPLICATION:
	    dprint(10, (stderr, "msgview_part2canvas(): application\n"));
            msgview_application2canvas (message, body, fp, group);
            break;

	case TYPEIMAGE:
	    dprint(10, (stderr, "msgview_part2canvas(): immage\n"));
            msgview_image2canvas (message, body, fp, group);
            break;

	case TYPEMESSAGE:
	    dprint(10, (stderr, "msgview_part2canvas(): message\n"));
            msgview_message2canvas (message, body, fp, group);
	    dprint(10, (stderr, "msgview_part2canvas(): message - part end\n"));
            break;
	    
	case TYPEMULTIPART:
	    dprint(10, (stderr, "msgview_part2canvas(): multipart\n"));
            msgview_multipart2canvas (message, body, fp, group);
	    dprint(10, (stderr, "msgview_part2canvas(): multipart - part end\n"));
            break;

	case TYPETEXT:
	    dprint(10, (stderr, "msgview_part2canvas(): text\n"));
            msgview_mimetext2canvas (message, body, fp, group);
            break;

	case TYPEVIDEO:
	    dprint(10, (stderr, "msgview_part2canvas(): video\n"));
            msgview_video2canvas (message, body, fp, group);
            break;
    }
}

static gboolean msgview_content2canvas (Message * message, GnomeCanvasGroup * group){

    if (message->content)
	msgview_part2canvas(message, message->content, message->mailbox->fp, group);

    return TRUE;
}

static gchar *save_message_part (Message * message, Body* body){

    char *filename;
    State s;

    s.fpin = fopen (message->mailbox->path, "r");
    s.fpout = utils_mktemp(&filename);
    s.prefix = 0;

    fseek (s.fpin, body->offset, 0);

    message_decode_attachment (body, &s);
    fclose (s.fpin);
    fclose (s.fpout);

    return filename;
}


SaveFileInfo *save_file_info_new (GtkWidget * widget, Message *message, Body *body){

    SaveFileInfo *save;

    save = g_malloc (sizeof(SaveFileInfo));

    save->file_entry = widget;
    save->msg = message;
    save->body = body;

    return save;
}


static void save_mime_part(GtkObject *obj, SaveFileInfo *info){

    gchar *filename;
    GtkWidget *file_entry = info->file_entry;
    char msg_filename[PATH_MAX + 1];
    State s;

    s.fpin = fopen (info->msg->mailbox->path, "r");

    if (!s.fpin || ferror (s.fpin)) {
        char msg[1024];
        GtkWidget *msgbox;

        snprintf (msg, 1023, _ (" Open of %s failed:%s "), msg_filename, strerror (errno));
        msgbox = gnome_message_box_new (msg, "Error", _ ("Ok"), NULL);
        gtk_window_set_modal (GTK_WINDOW (msgbox), TRUE);
        gnome_dialog_run (GNOME_DIALOG (msgbox));
        return;
    }

    filename = gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry))));
    s.prefix = 0;
    s.fpout = fopen (filename, "w");
    fseek (s.fpin, info->body->offset, 0);

    if (!s.fpout) {
        char msg[1024];
        GtkWidget *msgbox;

        snprintf (msg, 1023, _ (" Open of %s failed:%s "), filename, strerror (errno));
        msgbox = gnome_message_box_new (msg, "Error", _ ("Ok"), NULL);
        gtk_window_set_modal (GTK_WINDOW (msgbox), TRUE);
        gnome_dialog_run (GNOME_DIALOG (msgbox));
        return;
    }

    message_decode_attachment (info->body, &s);

    fclose (s.fpin);
    fclose (s.fpout);
}


