/*
 * Glacier - another Gnome email client.
 * Copyright (C) 1999 Lubomir Gelo
 *
 * Glacier uses code from other GPL-ed projects. See THANKS for 
 * credits and copyright info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "includes.h"

#define NONULL(x) x?x:""
#define IsPrint(c) (isprint((unsigned char)(c)) || (unsigned char)(c) >= 0xa0)

typedef void encode_t (char *, size_t, const unsigned char *);

const char *BodyTypes[] = { "x-unknown", "audio", "application", "image", "message", "model", "multipart", "text", "video" };
const char *BodyEncodings[] = { "x-unknown", "7bit", "8bit", "quoted-printable", "base64", "binary" };

const char MimeSpecials[] = "@.,;<>[]\\\"()?/=";

char B64Chars[64] = {
  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
  'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
  'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', '+', '/'
};

int Index_hex[128] = {
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
     0, 1, 2, 3,  4, 5, 6, 7,  8, 9,-1,-1, -1,-1,-1,-1,
    -1,10,11,12, 13,14,15,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,10,11,12, 13,14,15,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1
};

int Index_64[128] = {
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
    -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
    52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1,-1,-1,-1,
    -1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
    15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
    -1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
    41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
};

static void q_encode_string (char *d, size_t dlen, const unsigned char *s){

    char charset[SHORT_STRING];
    size_t cslen, wordlen;
    char *wptr = d;

    snprintf (charset, sizeof (charset), "=?%s?Q?", strcasecmp ("us-ascii", NONULL(glacier_app.charset)) == 0 ? "unknown-8bit" : NONULL(glacier_app.charset));
    cslen = strlen (charset);

    strcpy (wptr, charset);
    wptr += cslen;
    wordlen = cslen;
    dlen -= cslen;

    dlen -= 3; /* save room for the word terminator */

    while (*s && dlen > 0) {

	if (wordlen >= 72){

            if (dlen < 4 + cslen)
		break;

            strcpy (wptr, "?=\n ");
            wptr += 4;
    	    dlen -= 4;
            strcpy (wptr, charset);
            wptr += cslen;
            wordlen = cslen;
            dlen -= cslen;
	}

	if (*s == ' '){
            *wptr++ = '_';
            wordlen++;
            dlen--;

	} else if ((*s & 0x80) || *s == '\t' || strchr (MimeSpecials, *s)) {
      
    	    if (wordlen >= 70) {

		if (dlen < 4 + cslen)
	            break;

		strcpy (wptr, "?=\n ");
		wptr += 4;
		dlen -= 4;

		strcpy (wptr, charset);
		wptr += cslen;
		wordlen = cslen;
		dlen -= cslen;
            }

            if (dlen < 3)
		break;

            sprintf (wptr, "=%02X", *s);
            wptr += 3;
            wordlen += 3;
            dlen -= 3;

	} else {
      
            *wptr++ = *s;
            wordlen++;
            dlen--;
	}

	s++;
    }

    strcpy (wptr, "?=");
}

static void b_encode_string (char *d, size_t dlen, const unsigned char *s){

    char charset[SHORT_STRING];
    char *wptr = d;
    int cslen;
    int wordlen;

    snprintf (charset, sizeof (charset), "=?%s?B?", NONULL(glacier_app.charset));
    cslen = strlen (charset);
    strcpy (wptr, charset);
    wptr += cslen;
    wordlen = cslen;
    dlen -= cslen;

    dlen -= 3; /* save room for the word terminator */

    while (*s && dlen >= 4) {

	if (wordlen >= 71){
      
            if (dlen < 4 + cslen)
		break;

            strcpy (wptr, "?=\n ");
            wptr += 4;
            dlen -= 4;

            strcpy (wptr, charset);
            wptr += cslen;
            wordlen = cslen;
            dlen -= cslen;
	}

        *wptr++ = B64Chars[ (*s >> 2) & 0x3f ];
        *wptr++ = B64Chars[ ((*s & 0x3) << 4) | ((*(s+1) >> 4) & 0xf) ];
        s++;
    
	if (*s) {
            *wptr++ = B64Chars[ ((*s & 0xf) << 2) | ((*(s+1) >> 6) & 0x3) ];
            s++;

            if (*s) {
		*wptr++ = B64Chars[ *s & 0x3f ];
		s++;
            } else
		*wptr++ = '=';
	} else {

            *wptr++ = '=';
            *wptr++ = '=';
	}

	wordlen += 4;
	dlen -= 4;
    }

    strcpy (wptr, "?=");
}

void rfc2047_encode_string (char *d, size_t dlen, const unsigned char *s){

    int count = 0;
    int len;
    const unsigned char *p = s;
    encode_t *encoder;

    /* First check to see if there are any 8-bit characters */

    for (; *p; p++) {

	if (*p & 0x80)
            count++;
	else if (*p == '=' && *(p+1) == '?') {
            count += 2;
            p++;
	}
    }

    if (!count) {
	strfcpy (d, (const char *)s, dlen);
	return;
    }

    if (strcasecmp("us-ascii", NONULL(glacier_app.charset)) == 0 || strncasecmp("iso-8859", NONULL(glacier_app.charset), 8) == 0)
	encoder = q_encode_string;
    else {

	/* figure out which encoding generates the most compact representation */
	len = strlen ((char *) s);

	if ((count * 2) + len <= (4 * len) / 3)
            encoder = q_encode_string;
	else
            encoder = b_encode_string;
    }

    /* Hack to pull the Re: and Fwd: out of the encoded word for better
       handling by agents which do not support RFC2047.  */
  
    if (!strncasecmp ("re: ", (char *) s, 4)) {
	strncpy (d, (char *) s, 4);
	d += 4;
	dlen -= 4;
	s += 4;
    } else if (!strncasecmp ("fwd: ", (char *) s, 5)) {
	strncpy (d, (char *) s, 5);
	d += 5;
	dlen -= 5;
	s += 5;
    }

    (*encoder) (d, dlen, s);
}

void rfc2047_encode_addrlist (GList *list){

    GList 	*p = g_list_first(list);
    Address 	*addr;
    gchar	buffer[STRING];

    while (p){
    
	addr = (Address *)(p->data);
	if (addr->person != NULL){

            rfc2047_encode_string (buffer, sizeof (buffer), (const unsigned char *)addr->person);

	    g_free(addr->person);
	    addr->person = g_strdup(buffer);
	}

        p = g_list_next(p);
    }
}

static int rfc2047_decode_word(char *d, const char *s, size_t len){

    char *p = g_strdup(s);
    char *pp = p;
    char *pd = d;
    int enc = 0, filter = 0, count = 0, c1, c2, c3, c4;

    while ((pp = strtok (pp, "?")) != NULL) {

	count++;
	switch (count){

            case 2:
		if (strcasecmp (pp, NONULL(glacier_app.charset)) != 0)
	            filter = 1;
	        break;

          case 3:
	        if (toupper (*pp) == 'Q')
	            enc = ENCQUOTEDPRINTABLE;
		else if (toupper (*pp) == 'B')
	            enc = ENCBASE64;
		else
	            return (-1);
		break;

          case 4:
		if (enc == ENCQUOTEDPRINTABLE){
	  
	            while (*pp && len > 0) {

			if (*pp == '_') {
	                    *pd++ = ' ';
	                    len--;
			} else if (*pp == '=') {

	                    *pd++ = (hexval(pp[1]) << 4) | hexval(pp[2]);
	                    len--;
	                    pp += 2;

			} else {

	                    *pd++ = *pp;
	                    len--;
			}

			pp++;
		    }
	            *pd = 0;
		} else if (enc == ENCBASE64) {

	            while (*pp && len > 0) {

			c1 = base64val(pp[0]);
			c2 = base64val(pp[1]);

	    		*pd++ = (c1 << 2) | ((c2 >> 4) & 0x3);

			if (--len == 0) 
			    break;
	    
			if (pp[2] == '=') 
			    break;

			c3 = base64val(pp[2]);

			*pd++ = ((c2 & 0xf) << 4) | ((c3 >> 2) & 0xf);

			if (--len == 0)
	                    break;

			if (pp[3] == '=')
	                    break;

			c4 = base64val(pp[3]);   
			*pd++ = ((c3 & 0x3) << 6) | c4;

			if (--len == 0)
	                    break;

			pp += 4;
	          }

	          *pd = 0;
	    }
	    break;
	}

	pp = 0;
    }

    g_free ((void **) &p);

    if (filter) {

	pd = d;

	while (*pd) {

            if (! IsPrint(*pd))
		*pd = '?';
            pd++;
	}
    }

    return (0);
}

/* try to decode anything that looks like a valid RFC2047 encoded
 * header field, ignoring RFC822 parsing rules
 */
void rfc2047_decode(char *d, const char *s, size_t dlen){

    const char *p, *q;
    size_t n;
    int found_encoded = 0;

    dlen--; /* save room for the terminal nul */

    while (*s && dlen > 0) {
    
	if ((p = strstr (s, "=?")) == NULL || (q = strchr (p + 2, '?')) == NULL || (q = strchr (q + 1, '?')) == NULL ||	(q = strstr (q + 1, "?=")) == NULL) {

            /* no encoded words */
            if (d != s)
		strfcpy (d, s, dlen + 1);
            return;
	}

	if (p != s) {

            n = (size_t) (p - s);

            /* ignore spaces between encoded words */
            if (!found_encoded || strspn (s, " \t\r\n") != n) {

		if (n > dlen)
	            n = dlen;
	
		if (d != s)
	            memcpy (d, s, n);
	
		d += n;
		dlen -= n;
            }
	}

	rfc2047_decode_word (d, p, dlen);

	found_encoded = 1;
	s = q + 2;
	n = strlen (d);
	dlen -= n;
	d += n;
    }

    *d = 0;
}

void rfc2047_decode_addrlist (GList *list) {

    GList 	*p = g_list_first(list);
    Address 	*addr;

    while (p){
    
	addr = (Address *)(p->data);
	if (addr->person && strstr (addr->person, "=?") != NULL)
            rfc2047_decode (addr->person, addr->person, strlen (addr->person) + 1);

	p = g_list_next(p);
    }
}
