/*
 * Glacier - another Gnome email client.
 * Copyright (C) 1999 Lubomir Gelo
 *
 * Glacier uses code from other GPL-ed projects. See THANKS for 
 * credits and copyright info.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "includes.h"

static GlacierPrefs *glacier_prefs = NULL;


/* notebook pages */
static GtkWidget *prefs_create_identity_page (void);
static GtkWidget *prefs_create_display_page (void);
static GtkWidget *prefs_create_mailboxes_page (void);

static void prefs_dialog_apply (GnomePropertyBox * dialog, gint page, GlacierPrefs *glacier_prefs);
static void prefs_dialog_cancel (void);
static void prefs_dialog_set (void);

/* callbacks */
static void on_prefs_modified (GtkWidget *, GnomePropertyBox *);


guint toolbar_type[NUM_TOOLBAR_MODES] = {

    GTK_TOOLBAR_TEXT,
    GTK_TOOLBAR_ICONS,
    GTK_TOOLBAR_BOTH
};

gchar *toolbar_type_label[NUM_TOOLBAR_MODES] = {

    N_("Text"),
    N_("Icons"),
    N_("Both"),
};

void prefs_set_defaults(void){

    gchar dummy_path[STRING];
    gchar *dummy_string;

    glacier_app.app_width = 700;
    glacier_app.app_height = 500;

    glacier_app.msglist_width = 480;
    glacier_app.msglist_height = 200;

    glacier_app.mbtree_width = 220;
    glacier_app.mbtree_height = 500;

    glacier_app.msgview_width = 480;
    glacier_app.msgview_height = 300;

    glacier_app.layout = APP_LAYOUT_C;
    glacier_app.toolbar_style = GTK_TOOLBAR_BOTH;

    G_FREE(glacier_app.config_version);
    glacier_app.config_version = g_strdup(VERSION);

    G_FREE(glacier_app.mail_dir);
    g_snprintf(dummy_path, STRING, "%s/Mail", g_get_home_dir());
    glacier_app.mail_dir = g_strdup(dummy_path);
    
    G_FREE(glacier_app.inbox);
    glacier_app.inbox = g_strdup(getenv("MAIL"));

    G_FREE(glacier_app.outbox);
    g_snprintf(dummy_path, STRING, "%s/Outbox", glacier_app.mail_dir);
    glacier_app.outbox = g_strdup(dummy_path);

    G_FREE(glacier_app.drafts);
    g_snprintf(dummy_path, STRING, "%s/Drafts", glacier_app.mail_dir);
    glacier_app.drafts = g_strdup(dummy_path);

    G_FREE(glacier_app.trash);
    g_snprintf(dummy_path, STRING, "%s/Trash", glacier_app.mail_dir);
    glacier_app.trash = g_strdup(dummy_path);

    G_FREE(glacier_app.real_name);
    dummy_string = g_get_real_name();
    
    if (dummy_string != NULL){

        char *p;
        p = strchr (dummy_string, ',');
        if (p != NULL)
	    *p = '\0';

	glacier_app.real_name = g_strdup(dummy_string);
    }

    G_FREE(glacier_app.signature);
    g_snprintf(dummy_path, STRING, "%s/.signature", g_get_home_dir());
    glacier_app.signature = g_strdup(dummy_path);

    glacier_app.charset = "iso-8859-1";
}

void prefs_save(void){

    gnome_config_set_string("/glacier/config/version", VERSION);

    gnome_config_set_string("/glacier/mailboxes/maildir", glacier_app.mail_dir);
    gnome_config_set_string("/glacier/mailboxes/inbox", glacier_app.inbox);    
    gnome_config_set_string("/glacier/mailboxes/outbox", glacier_app.outbox);    
    gnome_config_set_string("/glacier/mailboxes/drafts", glacier_app.drafts);    
    gnome_config_set_string("/glacier/mailboxes/trash", glacier_app.trash);    


    gnome_config_set_int("/glacier/ui/layout", glacier_app.layout);
    gnome_config_set_int("/glacier/ui/app_width", glacier_app.app_width);
    gnome_config_set_int("/glacier/ui/app_height", glacier_app.app_height);
    gnome_config_set_int("/glacier/ui/mbtree_width", glacier_app.mbtree_width);
    gnome_config_set_int("/glacier/ui/mbtree_height", glacier_app.mbtree_height);
    gnome_config_set_int("/glacier/ui/msglist_width", glacier_app.msglist_width);
    gnome_config_set_int("/glacier/ui/msglist_height", glacier_app.msglist_height);
    gnome_config_set_int("/glacier/ui/msgview_width", glacier_app.msgview_width);
    gnome_config_set_int("/glacier/ui/msgview_height", glacier_app.msgview_height);

    gnome_config_sync();
}

void prefs_free(void){

    G_FREE(glacier_app.config_version);
    G_FREE(glacier_app.mail_dir);
    G_FREE(glacier_app.inbox);

    G_FREE(glacier_app.real_name);
    G_FREE(glacier_app.email_address);
    G_FREE(glacier_app.reply_to_address);
    G_FREE(glacier_app.signature);
}

int prefs_check_config_version(void){

    gboolean default_used;
    gint result;

    glacier_app.config_version = gnome_config_get_string_with_default ("/glacier/config/version=0.0.0", &default_used);
    
    if (!strcmp(glacier_app.config_version, "0.0.0"))
	return GLACIER_NOT_CONFIGURED;

    result = strcmp(glacier_app.config_version, VERSION);
    
    if (result == 0)
	return GLACIER_CONFIG_VERSION_OK;
    else if (result < 0)
	return GLACIER_CONFIG_IS_OLD;
    else
	return GLACIER_CONFIG_IS_NEWER;
}

void prefs_load(void) {

    gboolean default_used;
    gchar *param;

    prefs_set_defaults();

    param = gnome_config_get_string_with_default("/glacier/mailboxes/maildir", &default_used);
    
    if (!default_used && param){
	G_FREE(glacier_app.mail_dir);
	glacier_app.mail_dir = g_strdup(param);
    }
    
    param = gnome_config_get_string_with_default("/glacier/mailboxes/inbox", &default_used);    

    if (!default_used && param){
	G_FREE(glacier_app.inbox);
	glacier_app.inbox = g_strdup(param);
    }

    param = gnome_config_get_string_with_default("/glacier/mailboxes/outbox", &default_used);    

    if (!default_used && param){
	G_FREE(glacier_app.outbox);
	glacier_app.outbox = g_strdup(param);
    }

    param = gnome_config_get_string_with_default("/glacier/mailboxes/drafts", &default_used);    

    if (!default_used && param){
	G_FREE(glacier_app.drafts);
	glacier_app.drafts = g_strdup(param);
    }

    param = gnome_config_get_string_with_default("/glacier/mailboxes/trash", &default_used);    

    if (!default_used && param){
	G_FREE(glacier_app.trash);
	glacier_app.trash = g_strdup(param);
    }


    glacier_app.layout = gnome_config_get_int_with_default("/glacier/ui/layout=1", &default_used);
    glacier_app.app_width = gnome_config_get_int_with_default("/glacier/ui/app_width=700", &default_used);
    glacier_app.app_height = gnome_config_get_int_with_default("/glacier/ui/app_height=500", &default_used);
    glacier_app.mbtree_width = gnome_config_get_int_with_default("/glacier/ui/mbtree_width=220", &default_used);
    glacier_app.mbtree_height = gnome_config_get_int_with_default("/glacier/ui/mbtree_height=500", &default_used);
    glacier_app.msglist_width = gnome_config_get_int_with_default("/glacier/ui/msglist_width=480", &default_used);
    glacier_app.msglist_height = gnome_config_get_int_with_default("/glacier/ui/msglist_height=200", &default_used);
    glacier_app.msgview_width = gnome_config_get_int_with_default("/glacier/ui/msgview_width=480", &default_used);
    glacier_app.msgview_height = gnome_config_get_int_with_default("/glacier/ui/msgview_height=300", &default_used);

}



void prefs_dialog_open (void){

//    static GnomeHelpMenuEntry help_entry = { NULL, "properties" };
    GtkWidget *label;
    gint i;

    /* one and only prefs dialog */
    if (glacier_prefs){
        gdk_window_raise (GTK_WIDGET (GNOME_DIALOG (glacier_prefs->dialog))->window);
        return;
    }

    glacier_prefs = g_malloc (sizeof (GlacierPrefs));

    glacier_prefs->dialog = GNOME_PROPERTY_BOX (gnome_property_box_new ());
    gtk_window_set_title(GTK_WINDOW(glacier_prefs->dialog), _("Glacier Preferences"));

    gnome_dialog_set_parent(GNOME_DIALOG(glacier_prefs->dialog), GTK_WINDOW(glacier_app.app));

    gtk_signal_connect (GTK_OBJECT (glacier_prefs->dialog), "destroy", GTK_SIGNAL_FUNC (prefs_dialog_cancel), glacier_prefs);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->dialog), "delete_event", GTK_SIGNAL_FUNC (gtk_false), NULL);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->dialog), "apply", GTK_SIGNAL_FUNC (prefs_dialog_apply), glacier_prefs);

//    help_entry.name = gnome_app_id;
//    gtk_signal_connect (GTK_OBJECT (glacier_prefs->dialog), "help", GTK_SIGNAL_FUNC (gnome_help_dialog_display), &help_entry);

    /* identity page */
    label = gtk_label_new( _("Identity"));
    gtk_notebook_append_page ( GTK_NOTEBOOK (GNOME_PROPERTY_BOX (glacier_prefs->dialog)->notebook), prefs_create_identity_page (), label);

    /* mailboxes page */
    label = gtk_label_new( _("Mailboxes"));
    gtk_notebook_append_page ( GTK_NOTEBOOK (GNOME_PROPERTY_BOX (glacier_prefs->dialog)->notebook), prefs_create_mailboxes_page (), label);

    /* display page */
    label = gtk_label_new( _("Display"));
    gtk_notebook_append_page (GTK_NOTEBOOK (GNOME_PROPERTY_BOX (glacier_prefs->dialog)->notebook), prefs_create_display_page (), label);

    prefs_dialog_set();
  
    for (i = 0; i < NUM_TOOLBAR_MODES; i++) {
        gtk_signal_connect (GTK_OBJECT (glacier_prefs->toolbar_type[i]), "clicked", on_prefs_modified, glacier_prefs->dialog);
    }

    gtk_signal_connect (GTK_OBJECT (glacier_prefs->real_name), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->email), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->replyto), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->signature), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);

    gtk_signal_connect (GTK_OBJECT (glacier_prefs->inbox), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->outbox), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->drafts), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->trash), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);
    gtk_signal_connect (GTK_OBJECT (glacier_prefs->mail_dir), "changed", GTK_SIGNAL_FUNC (on_prefs_modified), glacier_prefs->dialog);

    gtk_widget_show_all (GTK_WIDGET (glacier_prefs->dialog));
}

static void prefs_dialog_cancel (void){

    gtk_widget_destroy (GTK_WIDGET (glacier_prefs->dialog));
    G_FREE(glacier_prefs);
}

static void prefs_dialog_apply (GnomePropertyBox * dialog, gint page, GlacierPrefs *glacier_prefs){

    gint i;
  
    /* identity page */

    G_FREE(glacier_app.email_address);
    glacier_app.email_address = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->email)));

    G_FREE(glacier_app.real_name);
    glacier_app.real_name = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->real_name)));

    G_FREE(glacier_app.reply_to_address);
    glacier_app.reply_to_address = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->replyto)));

    G_FREE(glacier_app.signature);
    glacier_app.signature = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->signature)));

    /* mailboxes page */

    G_FREE(glacier_app.inbox);
    glacier_app.inbox = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->inbox)));

    G_FREE(glacier_app.outbox);
    glacier_app.outbox = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->outbox)));

    G_FREE(glacier_app.drafts);
    glacier_app.drafts = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->drafts)));

    G_FREE(glacier_app.trash);
    glacier_app.trash = g_strdup (gtk_entry_get_text(GTK_ENTRY(glacier_prefs->trash)));

    /* display page */
    for (i = 0; i < NUM_TOOLBAR_MODES; i++){
        if (GTK_TOGGLE_BUTTON (glacier_prefs->toolbar_type[i])->active){
	    glacier_app.toolbar_style = toolbar_type[i];
	    break;
	}
    }	

    prefs_save();
}


void prefs_dialog_set(void){

    gint i;

    for (i = 0; i < NUM_TOOLBAR_MODES; i++){
        if (glacier_app.toolbar_style == toolbar_type[i]){
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glacier_prefs->toolbar_type[i]), TRUE);
	    break;
        }
    }

    gtk_entry_set_text(GTK_ENTRY(glacier_prefs->real_name), glacier_app.real_name);
    gtk_entry_set_text(GTK_ENTRY(glacier_prefs->email), glacier_app.email_address);
    gtk_entry_set_text(GTK_ENTRY(glacier_prefs->replyto), glacier_app.reply_to_address);
    gtk_entry_set_text (GTK_ENTRY (glacier_prefs->signature), glacier_app.signature);

    gtk_entry_set_text (GTK_ENTRY (glacier_prefs->mail_dir), glacier_app.mail_dir);
    gtk_entry_set_text (GTK_ENTRY (glacier_prefs->inbox), glacier_app.inbox);
    gtk_entry_set_text (GTK_ENTRY (glacier_prefs->outbox), glacier_app.outbox);
    gtk_entry_set_text (GTK_ENTRY (glacier_prefs->drafts), glacier_app.drafts);
    gtk_entry_set_text (GTK_ENTRY (glacier_prefs->trash), glacier_app.trash);
}


/* identity notebook page  */

static GtkWidget *prefs_create_identity_page (void){

    GtkWidget *vbox;
    GtkWidget *table;
    GtkWidget *label;
    GtkWidget *signature;

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);

    table = gtk_table_new(4, 2, FALSE);
    gtk_box_pack_start(GTK_BOX (vbox), table, FALSE, FALSE, 5);

    /* your name */
    label = gtk_label_new(_("Your name:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 10, 10);

    glacier_prefs->real_name = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), glacier_prefs->real_name, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    /* email address */
    label = gtk_label_new (_ ("E-Mail address:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 10, 10);

    glacier_prefs->email = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), glacier_prefs->email, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    /* reply-to address */
    label = gtk_label_new (_ ("Reply-to address:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 10, 10);

    glacier_prefs->replyto = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), glacier_prefs->replyto, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    /* signature */
    label = gtk_label_new (_ ("Signature:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 10, 10);

    signature = gnome_file_entry_new ("Signature", "Signature");
    gtk_table_attach (GTK_TABLE (table), signature, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    glacier_prefs->signature = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (signature));

    return vbox;
}

/* identity mailboxes page  */

static GtkWidget *prefs_create_mailboxes_page (void){

    GtkWidget *vbox;
    GtkWidget *table;
    GtkWidget *label;
    GtkWidget *file_entry;

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER (vbox), 10);

    table = gtk_table_new(5, 2, FALSE);
    gtk_box_pack_start(GTK_BOX (vbox), table, FALSE, FALSE, 5);

    /* mail dir */
    label = gtk_label_new(_("Local mail dir:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 10, 10);

    file_entry = gnome_file_entry_new ("Maildir", "Maildir");
    gtk_table_attach (GTK_TABLE (table), file_entry, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    glacier_prefs->mail_dir = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry));

    /* inbox */
    label = gtk_label_new (_ ("Inbox:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 10, 10);

    file_entry = gnome_file_entry_new ("Inbox", "Inbox");
    gtk_table_attach (GTK_TABLE (table), file_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    glacier_prefs->inbox = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry));

    /* outbox */
    label = gtk_label_new (_ ("Outbox:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 10, 10);

    file_entry = gnome_file_entry_new ("Outbox", "Outbox");
    gtk_table_attach (GTK_TABLE (table), file_entry, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    glacier_prefs->outbox = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry));

    /* drafts */
    label = gtk_label_new( _("Drafts:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 10, 10);

    file_entry = gnome_file_entry_new ("Drafts", "Drafts");
    gtk_table_attach (GTK_TABLE (table), file_entry, 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    glacier_prefs->drafts = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry));

    /* drafts */
    label = gtk_label_new( _("Trash:"));
    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 10, 10);

    file_entry = gnome_file_entry_new ("Trash", "Trash");
    gtk_table_attach (GTK_TABLE (table), file_entry, 1, 2, 4, 5, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 10);

    glacier_prefs->trash = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry));

    return vbox;
}


/* display notebook page */

static GtkWidget *prefs_create_display_page (){

    GtkWidget *vbox, *vbox1;
    GtkWidget *frame;
    GSList *group;
    gint i;

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);

    /* Toolbars */
    frame = gtk_frame_new (_ ("Toolbars"));
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 5);

    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (vbox1));

    group = NULL;
    for (i = 0; i < NUM_TOOLBAR_MODES; i++) {
        glacier_prefs->toolbar_type[i] = GTK_RADIO_BUTTON (gtk_radio_button_new_with_label (group, _(toolbar_type_label[i])));
        gtk_box_pack_start (GTK_BOX (vbox1), GTK_WIDGET (glacier_prefs->toolbar_type[i]), TRUE, TRUE, 2);
        group = gtk_radio_button_group (glacier_prefs->toolbar_type[i]);
    }

    return vbox;
}


static void on_prefs_modified (GtkWidget * widget, GnomePropertyBox * dialog){

    gnome_property_box_changed (dialog);
}

